-- ==========================================================================================
-- Project:		orgAnice.DSGVO
-- Author:		Falk Wegener, Xuan Guo
-- Last Modified:	2018-09-26
-- Description:	Creates SQL Objects for the DSGVO AddOn, wich cannot be added easily through a
--				orgAnice Update database project.
-- ==========================================================================================
declare @dbName nvarchar(255)
set @dbName = 'use [<<SQL-ORG-DB-NAME>>]'; 

EXEC sp_executesql @dbName;

GO
-- Part 01 - Adding needed schemas and tables, only if they do not exists. This includes PRJ-tables too, because they are included within the script

BEGIN TRANSACTION
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating schemas'
GO
IF SCHEMA_ID(N'cstm') IS NULL
EXEC sp_executesql N'CREATE SCHEMA [cstm]
AUTHORIZATION [dbo]'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
IF SCHEMA_ID(N'org') IS NULL
EXEC sp_executesql N'CREATE SCHEMA [org]
AUTHORIZATION [dbo]'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
IF SCHEMA_ID(N'privacy') IS NULL
EXEC sp_executesql N'CREATE SCHEMA [privacy]
AUTHORIZATION [dbo]'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [org].[Log]'
GO
IF OBJECT_ID(N'[org].[Log]', 'U') IS NULL
CREATE TABLE [org].[Log]
(
[__Org_RecordOID] [uniqueidentifier] NULL ROWGUIDCOL CONSTRAINT [__OrgD_Log___Org_RecordOID] DEFAULT (newsequentialid()),
[__Org_Tenants] [int] NOT NULL CONSTRAINT [__OrgD_Log___Org_Tenants] DEFAULT ((2)),
[ID] [int] NOT NULL IDENTITY(1, 1),
[TimeStamp] [datetime] NOT NULL CONSTRAINT [DF__Lo__TimeS__15C92877] DEFAULT (getdate()),
[Source] [nvarchar] (50) COLLATE Latin1_General_CI_AS NOT NULL,
[Type] [nvarchar] (20) COLLATE Latin1_General_CI_AS NOT NULL,
[Message] [nvarchar] (100) COLLATE Latin1_General_CI_AS NOT NULL,
[Data] [nvarchar] (max) COLLATE Latin1_General_CI_AS NULL,
[CreatedOn] [datetime] NULL CONSTRAINT [DF__Lo__Creat__16BD4CB0] DEFAULT (getdate()),
[CreatedBy] [nvarchar] (30) COLLATE Latin1_General_CI_AS NULL CONSTRAINT [DF__Lo__Creat__17B170E9] DEFAULT (suser_sname()),
[ModifiedOn] [datetime] NULL CONSTRAINT [DF__Lo__Modif__18A59522] DEFAULT (getdate()),
[ModifiedBy] [nvarchar] (30) COLLATE Latin1_General_CI_AS NULL CONSTRAINT [DF__Lo__Modif__1999B95B] DEFAULT (suser_sname()),
[TableName] [nvarchar] (128) COLLATE Latin1_General_CI_AS NULL,
[RecordNo] [int] NULL,
[Description] [nvarchar] (500) COLLATE Latin1_General_CI_AS NULL,
[LogID] AS (left(format([RecordNo],'d8'),(8))),
[RecordID] [nvarchar] (8) COLLATE Latin1_General_CI_AS NULL
)
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating primary key [PK_doc_MonitorLog_Id] on [org].[Log]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.indexes WHERE name = N'PK_doc_MonitorLog_Id' AND object_id = OBJECT_ID(N'[org].[Log]'))
ALTER TABLE [org].[Log] ADD CONSTRAINT [PK_doc_MonitorLog_Id] PRIMARY KEY CLUSTERED  ([ID])
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [privacy].[__Org_Documents_ExternalFileNamesFromDeletedRecords]'
GO
IF OBJECT_ID(N'[privacy].[__Org_Documents_ExternalFileNamesFromDeletedRecords]', 'U') IS NULL
CREATE TABLE [privacy].[__Org_Documents_ExternalFileNamesFromDeletedRecords]
(
[__Org_RecordOID] [uniqueidentifier] NULL ROWGUIDCOL CONSTRAINT [__OrgD___Org_Documents_ExternalFileNamesFromDeletedRecords___Org_RecordOID] DEFAULT (newsequentialid()),
[__Org_RecordNo] [int] NOT NULL IDENTITY(1, 1),
[ID] AS (right('0000000'+CONVERT([varchar](8),[__Org_RecordNo],(0)),(8))),
[DocOid] [uniqueidentifier] NOT NULL,
[DocLink] [nvarchar] (max) COLLATE Latin1_General_CI_AS NULL,
[DocExtension] [nvarchar] (256) COLLATE Latin1_General_CI_AS NULL,
[psDeleteCommand] [nvarchar] (max) COLLATE Latin1_General_CI_AS NULL,
[AngelegtAm] [datetime] NULL,
[AngelegtDurch] [nvarchar] (31) COLLATE Latin1_General_CI_AS NULL,
[ModifiziertAm] [datetime] NULL,
[ModifiziertDurch] [nvarchar] (31) COLLATE Latin1_General_CI_AS NULL,
[DeletedOn] [datetime] NULL,
[DeletedBy] [nvarchar] (31) COLLATE Latin1_General_CI_AS NULL,
[ExternalFileDeleteSuccessfull] [bit] NULL,
[ExternalFileDeletedOn] [datetime] NULL,
[ExternalFileDeletedBy] [nvarchar] (31) COLLATE Latin1_General_CI_AS NULL,
[ExternalFileDeleteMessage] [nvarchar] (max) COLLATE Latin1_General_CI_AS NULL
)
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating primary key [PK____Org_Do__5EC50C0EDDC24798] on [privacy].[__Org_Documents_ExternalFileNamesFromDeletedRecords]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.indexes WHERE name = N'PK____Org_Do__5EC50C0EDDC24798' AND object_id = OBJECT_ID(N'[privacy].[__Org_Documents_ExternalFileNamesFromDeletedRecords]'))
ALTER TABLE [privacy].[__Org_Documents_ExternalFileNamesFromDeletedRecords] ADD CONSTRAINT [PK____Org_Do__5EC50C0EDDC24798] PRIMARY KEY CLUSTERED  ([__Org_RecordNo])
GO
IF @@ERROR <> 0 SET NOEXEC ON

GO
PRINT N'Adding constraints to [org].[Log]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.check_constraints WHERE object_id = OBJECT_ID(N'[org].[__OrgC_Log___Org_Tenants]', 'C') AND parent_object_id = OBJECT_ID(N'[org].[Log]', 'U'))
ALTER TABLE [org].[Log] ADD CONSTRAINT [__OrgC_Log___Org_Tenants] CHECK NOT FOR REPLICATION ((coalesce([Log].[__Org_Tenants],(0)&((-2)))<>(0)))
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Disabling constraints on [org].[Log]'
GO
IF EXISTS (SELECT 1 FROM sys.check_constraints WHERE object_id = OBJECT_ID(N'[org].[__OrgC_Log___Org_Tenants]', 'C') AND parent_object_id = OBJECT_ID(N'[org].[Log]', 'U'))
ALTER TABLE [org].[Log] NOCHECK CONSTRAINT [__OrgC_Log___Org_Tenants]
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [dbo].[DSGVOFeldKonfigurationen]'
GO
IF OBJECT_ID(N'[dbo].[DSGVOFeldKonfigurationen]', 'U') IS NULL
CREATE TABLE [dbo].[DSGVOFeldKonfigurationen]
(
[__Org_RecordOID] [uniqueidentifier] NULL ROWGUIDCOL CONSTRAINT [__OrgD_DSGVOFeldKonfigurationen___Org_RecordOID] DEFAULT (newsequentialid()),
[__Org_RecordNo] [int] NOT NULL IDENTITY(1, 1),
[__Org_Tenants] [int] NOT NULL CONSTRAINT [__OrgD_DSGVOFeldKonfigurationen___Org_Tenants] DEFAULT ((2)),
[ID] [nvarchar] (8) COLLATE Latin1_General_CI_AS NOT NULL CONSTRAINT [__OrgD_DSGVOFeldKonfigurationen_ID] DEFAULT (''),
[AngelegtAm] [datetime] NULL,
[AngelegtDurch] [nvarchar] (31) COLLATE Latin1_General_CI_AS NULL,
[ModifiziertAm] [datetime] NULL,
[ModifiziertDurch] [nvarchar] (31) COLLATE Latin1_General_CI_AS NULL,
[TabellenName] [nvarchar] (31) COLLATE Latin1_General_CI_AS NOT NULL CONSTRAINT [__OrgD_DSGVOFeldKonfigurationen_TabellenName] DEFAULT (''),
[FeldName] [nvarchar] (31) COLLATE Latin1_General_CI_AS NOT NULL CONSTRAINT [__OrgD_DSGVOFeldKonfigurationen_FeldName] DEFAULT (''),
[EigenschaftenName] [nvarchar] (31) COLLATE Latin1_General_CI_AS NULL,
[EigenschaftsWert] [nvarchar] (31) COLLATE Latin1_General_CI_AS NULL,
[SystemFeld] [bit] NULL,
[Bemerkung] [nvarchar] (max) COLLATE Latin1_General_CI_AS NULL,
[__Org__Eigenschaften] [nvarchar] (93) COLLATE Latin1_General_CI_AS NULL
)
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating primary key [PK__DSGVOFel__5EC50C0E54D2E7C0] on [dbo].[DSGVOFeldKonfigurationen]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.indexes WHERE is_primary_key= 1 AND  object_id = OBJECT_ID(N'[dbo].[DSGVOFeldKonfigurationen]'))
BEGIN
	IF NOT EXISTS (SELECT 1 FROM sys.indexes WHERE name = N'PK__DSGVOFel__5EC50C0E54D2E7C0' AND object_id = OBJECT_ID(N'[dbo].[DSGVOFeldKonfigurationen]'))
	ALTER TABLE [dbo].[DSGVOFeldKonfigurationen] ADD CONSTRAINT [PK__DSGVOFel__5EC50C0E54D2E7C0] PRIMARY KEY CLUSTERED  ([__Org_RecordNo])
END;
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating index [Eigenschaften] on [dbo].[DSGVOFeldKonfigurationen]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.indexes WHERE name = N'Eigenschaften' AND object_id = OBJECT_ID(N'[dbo].[DSGVOFeldKonfigurationen]'))
CREATE UNIQUE NONCLUSTERED INDEX [Eigenschaften] ON [dbo].[DSGVOFeldKonfigurationen] ([__Org__Eigenschaften])
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating index [__OrgS_RecordNo] on [dbo].[DSGVOFeldKonfigurationen]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.indexes WHERE name = N'__OrgS_RecordNo' AND object_id = OBJECT_ID(N'[dbo].[DSGVOFeldKonfigurationen]'))
CREATE NONCLUSTERED INDEX [__OrgS_RecordNo] ON [dbo].[DSGVOFeldKonfigurationen] ([__Org_RecordNo])
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating index [__OrgS_RecordOID] on [dbo].[DSGVOFeldKonfigurationen]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.indexes WHERE name = N'__OrgS_RecordOID' AND object_id = OBJECT_ID(N'[dbo].[DSGVOFeldKonfigurationen]'))
CREATE UNIQUE NONCLUSTERED INDEX [__OrgS_RecordOID] ON [dbo].[DSGVOFeldKonfigurationen] ([__Org_RecordOID])
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating index [Felder] on [dbo].[DSGVOFeldKonfigurationen]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.indexes WHERE name = N'Felder' AND object_id = OBJECT_ID(N'[dbo].[DSGVOFeldKonfigurationen]'))
CREATE NONCLUSTERED INDEX [Felder] ON [dbo].[DSGVOFeldKonfigurationen] ([FeldName])
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating index [ID] on [dbo].[DSGVOFeldKonfigurationen]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.indexes WHERE name = N'ID' AND object_id = OBJECT_ID(N'[dbo].[DSGVOFeldKonfigurationen]'))
CREATE NONCLUSTERED INDEX [ID] ON [dbo].[DSGVOFeldKonfigurationen] ([ID])
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating index [Tabellen] on [dbo].[DSGVOFeldKonfigurationen]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.indexes WHERE name = N'Tabellen' AND object_id = OBJECT_ID(N'[dbo].[DSGVOFeldKonfigurationen]'))
CREATE NONCLUSTERED INDEX [Tabellen] ON [dbo].[DSGVOFeldKonfigurationen] ([TabellenName])
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating trigger [dbo].[__OrgT_DSGVOFeldKonfigurationen_Redundant] on [dbo].[DSGVOFeldKonfigurationen]'
GO
IF OBJECT_ID(N'[dbo].[__OrgT_DSGVOFeldKonfigurationen_Redundant]', 'TR') IS NULL
EXEC sp_executesql N'CREATE TRIGGER [dbo].[__OrgT_DSGVOFeldKonfigurationen_Redundant]
ON [dbo].[DSGVOFeldKonfigurationen]
AFTER INSERT, UPDATE NOT FOR REPLICATION
AS 
SET NOCOUNT ON

DECLARE @ReplicaId nvarchar(3)
SET @ReplicaId = (SELECT [Value] FROM __Org_DbProps WHERE [Name] = ''ReplicaID'')
DECLARE @RecordNo int
DECLARE @C CURSOR
SET @C = CURSOR FORWARD_ONLY FOR
  SELECT [__Org_RecordNo]
  FROM inserted
  WHERE [__Org_RecordNo] NOT IN (SELECT [__Org_RecordNo] FROM deleted)
  ORDER BY [__Org_RecordNo]
FOR READ ONLY
OPEN @C
FETCH FROM @C INTO @RecordNo
WHILE @@FETCH_STATUS = 0
BEGIN
	UPDATE [DSGVOFeldKonfigurationen]
	SET [ID] = dbo.__Org_NextAutoCountValue((SELECT MAX([ID]) FROM [DSGVOFeldKonfigurationen] WHERE [ID] LIKE @ReplicaId + ''[0-9]%'' AND [__Org_RecordNo] <> @RecordNo), @ReplicaId, 8)
	WHERE [__Org_RecordNo] = @RecordNo
	FETCH FROM @C INTO @RecordNo
END
CLOSE @C
DEALLOCATE @C

IF UPDATE([TabellenName]) OR UPDATE([FeldName]) OR UPDATE([EigenschaftenName])
UPDATE [DSGVOFeldKonfigurationen]
SET __Org__Eigenschaften = SUBSTRING((((COALESCE("DSGVOFeldKonfigurationen"."TabellenName", '''') COLLATE Latin1_General_CS_AS)) + ((COALESCE("DSGVOFeldKonfigurationen"."FeldName", '''') COLLATE Latin1_General_CS_AS))) + ((COALESCE("DSGVOFeldKonfigurationen"."EigenschaftenName", '''') COLLATE Latin1_General_CS_AS)), 1, 93)
WHERE [DSGVOFeldKonfigurationen].[__Org_RecordNo] IN (SELECT inserted.[__Org_RecordNo] FROM inserted)

'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [dbo].[Blacklist]'
GO
IF OBJECT_ID(N'[dbo].[Blacklist]', 'U') IS NULL
CREATE TABLE [dbo].[Blacklist]
(
[__Org_RecordOID] [uniqueidentifier] NULL ROWGUIDCOL CONSTRAINT [__OrgD_Blacklist___Org_RecordOID] DEFAULT (newsequentialid()),
[__Org_RecordNo] [int] NOT NULL IDENTITY(1, 1),
[__Org_Tenants] [int] NOT NULL CONSTRAINT [__OrgD_Blacklist___Org_Tenants] DEFAULT ((2)),
[Vorname] [nvarchar] (200) COLLATE Latin1_General_CI_AS NULL,
[Name] [nvarchar] (200) COLLATE Latin1_General_CI_AS NULL,
[Zusatz] [nvarchar] (200) COLLATE Latin1_General_CI_AS NULL,
[Strasse] [nvarchar] (200) COLLATE Latin1_General_CI_AS NULL,
[PLZ] [nvarchar] (200) COLLATE Latin1_General_CI_AS NULL,
[Ort] [nvarchar] (200) COLLATE Latin1_General_CI_AS NULL,
[Land] [nvarchar] (200) COLLATE Latin1_General_CI_AS NULL,
[Loeschgrund] [nvarchar] (max) COLLATE Latin1_General_CI_AS NULL,
[ID] [nvarchar] (8) COLLATE Latin1_General_CI_AS NOT NULL CONSTRAINT [__OrgD_Blacklist_ID] DEFAULT (''),
[AngelegtAm] [datetime] NULL,
[AngelegtDurch] [nvarchar] (31) COLLATE Latin1_General_CI_AS NULL,
[ModifiziertAm] [datetime] NULL,
[ModifiziertDurch] [nvarchar] (31) COLLATE Latin1_General_CI_AS NULL,
[Email] [nvarchar] (200) COLLATE Latin1_General_CI_AS NULL,
[Archiv] [bit] NULL,
[ArchiviertVon] [nvarchar] (31) COLLATE Latin1_General_CI_AS NULL,
[ArchiviertAm] [datetime] NULL,
[__Org__Name] [nvarchar] (100) COLLATE Latin1_General_CI_AS NULL
)
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating primary key [PK__Blacklis__5EC50C0E0DD068EE] on [dbo].[Blacklist]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.indexes WHERE is_primary_key= 1 AND  object_id = OBJECT_ID(N'[dbo].[Blacklist]'))
BEGIN
	IF NOT EXISTS (SELECT 1 FROM sys.indexes WHERE name = N'PK__Blacklis__5EC50C0E0DD068EE' AND object_id = OBJECT_ID(N'[dbo].[Blacklist]'))
	ALTER TABLE [dbo].[Blacklist] ADD CONSTRAINT [PK__Blacklis__5EC50C0E0DD068EE] PRIMARY KEY CLUSTERED  ([__Org_RecordNo])
END;
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating index [Name] on [dbo].[Blacklist]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.indexes WHERE name = N'Name' AND object_id = OBJECT_ID(N'[dbo].[Blacklist]'))
CREATE NONCLUSTERED INDEX [Name] ON [dbo].[Blacklist] ([__Org__Name])
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating index [__OrgS_RecordNo] on [dbo].[Blacklist]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.indexes WHERE name = N'__OrgS_RecordNo' AND object_id = OBJECT_ID(N'[dbo].[Blacklist]'))
CREATE NONCLUSTERED INDEX [__OrgS_RecordNo] ON [dbo].[Blacklist] ([__Org_RecordNo])
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating index [__OrgS_RecordOID] on [dbo].[Blacklist]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.indexes WHERE name = N'__OrgS_RecordOID' AND object_id = OBJECT_ID(N'[dbo].[Blacklist]'))
CREATE UNIQUE NONCLUSTERED INDEX [__OrgS_RecordOID] ON [dbo].[Blacklist] ([__Org_RecordOID])
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating index [ID] on [dbo].[Blacklist]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.indexes WHERE name = N'ID' AND object_id = OBJECT_ID(N'[dbo].[Blacklist]'))
CREATE NONCLUSTERED INDEX [ID] ON [dbo].[Blacklist] ([ID])
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating trigger [dbo].[__OrgT_Blacklist_Redundant] on [dbo].[Blacklist]'
GO
IF OBJECT_ID(N'[dbo].[__OrgT_Blacklist_Redundant]', 'TR') IS NULL
EXEC sp_executesql N'CREATE TRIGGER [dbo].[__OrgT_Blacklist_Redundant]
ON [dbo].[Blacklist]
AFTER INSERT, UPDATE NOT FOR REPLICATION
AS 
SET NOCOUNT ON

DECLARE @ReplicaId nvarchar(3)
SET @ReplicaId = (SELECT [Value] FROM __Org_DbProps WHERE [Name] = ''ReplicaID'')
DECLARE @RecordNo int
DECLARE @C CURSOR
SET @C = CURSOR FORWARD_ONLY FOR
  SELECT [__Org_RecordNo]
  FROM inserted
  WHERE [__Org_RecordNo] NOT IN (SELECT [__Org_RecordNo] FROM deleted)
  ORDER BY [__Org_RecordNo]
FOR READ ONLY
OPEN @C
FETCH FROM @C INTO @RecordNo
WHILE @@FETCH_STATUS = 0
BEGIN
	UPDATE [Blacklist]
	SET [ID] = dbo.__Org_NextAutoCountValue((SELECT MAX([ID]) FROM [Blacklist] WHERE [ID] LIKE @ReplicaId + ''[0-9]%'' AND [__Org_RecordNo] <> @RecordNo), @ReplicaId, 8)
	WHERE [__Org_RecordNo] = @RecordNo
	FETCH FROM @C INTO @RecordNo
END
CLOSE @C
DEALLOCATE @C

IF UPDATE([Vorname]) OR UPDATE([Name])
UPDATE [Blacklist]
SET __Org__Name = SUBSTRING(UPPER((((COALESCE("Blacklist"."Name", '''') COLLATE Latin1_General_CS_AS)) + (('' '' COLLATE Latin1_General_CS_AS))) + ((COALESCE("Blacklist"."Vorname", '''') COLLATE Latin1_General_CS_AS))), 1, 100)
WHERE [Blacklist].[__Org_RecordNo] IN (SELECT inserted.[__Org_RecordNo] FROM inserted)

'
GO

IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating extended properties'
GO
BEGIN TRY
	IF NOT EXISTS (SELECT 1 FROM fn_listextendedproperty(N'MS_Description', 'SCHEMA', N'org', 'TABLE', N'Log', NULL, NULL))
EXEC sp_addextendedproperty N'MS_Description', N'Logging notifications and errors while executing sps or external code. Records can be related to the ID from the record in the related table.', 'SCHEMA', N'org', 'TABLE', N'Log', NULL, NULL
END TRY
BEGIN CATCH
	DECLARE @msg nvarchar(max);
	DECLARE @severity int;
	DECLARE @state int;
	SELECT @msg = ERROR_MESSAGE(), @severity = ERROR_SEVERITY(), @state = ERROR_STATE();
	RAISERROR(@msg, @severity, @state);

	SET NOEXEC ON
END CATCH
GO
BEGIN TRY
	IF NOT EXISTS (SELECT 1 FROM fn_listextendedproperty(N'RententionTimeInDays', 'SCHEMA', N'org', 'TABLE', N'Log', NULL, NULL))
EXEC sp_addextendedproperty N'RententionTimeInDays', N'30', 'SCHEMA', N'org', 'TABLE', N'Log', NULL, NULL
END TRY
BEGIN CATCH
	DECLARE @msg nvarchar(max);
	DECLARE @severity int;
	DECLARE @state int;
	SELECT @msg = ERROR_MESSAGE(), @severity = ERROR_SEVERITY(), @state = ERROR_STATE();
	RAISERROR(@msg, @severity, @state);

	SET NOEXEC ON
END CATCH
GO
BEGIN TRY
	IF NOT EXISTS (SELECT 1 FROM fn_listextendedproperty(N'RententionTimeInDays', 'SCHEMA', N'privacy', 'TABLE', N'__Org_Documents_ExternalFileNamesFromDeletedRecords', NULL, NULL))
EXEC sp_addextendedproperty N'RententionTimeInDays', N'30', 'SCHEMA', N'privacy', 'TABLE', N'__Org_Documents_ExternalFileNamesFromDeletedRecords', NULL, NULL
END TRY
BEGIN CATCH
	DECLARE @msg nvarchar(max);
	DECLARE @severity int;
	DECLARE @state int;
	SELECT @msg = ERROR_MESSAGE(), @severity = ERROR_SEVERITY(), @state = ERROR_STATE();
	RAISERROR(@msg, @severity, @state);

	SET NOEXEC ON
END CATCH
GO
BEGIN TRY
	IF NOT EXISTS (SELECT 1 FROM fn_listextendedproperty(N'MS_Description', 'SCHEMA', N'cstm', NULL, NULL, NULL, NULL))
EXEC sp_addextendedproperty N'MS_Description', N'schema for all customer related extensions within a database. All project related functions are stored in this schema.', 'SCHEMA', N'cstm', NULL, NULL, NULL, NULL
END TRY
BEGIN CATCH
	DECLARE @msg nvarchar(max);
	DECLARE @severity int;
	DECLARE @state int;
	SELECT @msg = ERROR_MESSAGE(), @severity = ERROR_SEVERITY(), @state = ERROR_STATE();
	RAISERROR(@msg, @severity, @state);

	SET NOEXEC ON
END CATCH
GO
BEGIN TRY
	IF NOT EXISTS (SELECT 1 FROM fn_listextendedproperty(N'MS_Description', 'SCHEMA', N'org', NULL, NULL, NULL, NULL))
EXEC sp_addextendedproperty N'MS_Description', N'schema for all orgAnice core sp and fn for a database. All essential functions, views, tables and sp are stored in this schema.', 'SCHEMA', N'org', NULL, NULL, NULL, NULL
END TRY
BEGIN CATCH
	DECLARE @msg nvarchar(max);
	DECLARE @severity int;
	DECLARE @state int;
	SELECT @msg = ERROR_MESSAGE(), @severity = ERROR_SEVERITY(), @state = ERROR_STATE();
	RAISERROR(@msg, @severity, @state);

	SET NOEXEC ON
END CATCH
GO
BEGIN TRY
	IF NOT EXISTS (SELECT 1 FROM fn_listextendedproperty(N'MS_Description', 'SCHEMA', N'privacy', NULL, NULL, NULL, NULL))
EXEC sp_addextendedproperty N'MS_Description', N'schema for all privacy protection related extensions within a database. All PDP (Private Data Protection) functions are stored in this schema.', 'SCHEMA', N'privacy', NULL, NULL, NULL, NULL
END TRY
BEGIN CATCH
	DECLARE @msg nvarchar(max);
	DECLARE @severity int;
	DECLARE @state int;
	SELECT @msg = ERROR_MESSAGE(), @severity = ERROR_SEVERITY(), @state = ERROR_STATE();
	RAISERROR(@msg, @severity, @state);

	SET NOEXEC ON
END CATCH
GO
COMMIT TRANSACTION
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
DECLARE @Success AS BIT
SET @Success = 1
SET NOEXEC OFF
IF (@Success = 1) PRINT 'The database update for part 01 succeeded'
ELSE BEGIN
	IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION
	PRINT 'The database update for part 01 failed'
END
GO

-- Part 03 - Update sps ufn and aid tables. Thouse objects will be replaced or created
SET NUMERIC_ROUNDABORT OFF
GO
SET ANSI_PADDING, ANSI_WARNINGS, CONCAT_NULL_YIELDS_NULL, ARITHABORT, QUOTED_IDENTIFIER, ANSI_NULLS ON
GO
SET XACT_ABORT ON
GO
SET TRANSACTION ISOLATION LEVEL Serializable
GO
BEGIN TRANSACTION
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Dropping extended properties'
GO
BEGIN TRY
	IF EXISTS (SELECT 1 FROM fn_listextendedproperty(N'MS_Description', 'SCHEMA', N'privacy', 'TABLE', N'CleanUpJobList_aid', NULL, NULL))
EXEC sp_dropextendedproperty N'MS_Description', 'SCHEMA', N'privacy', 'TABLE', N'CleanUpJobList_aid', NULL, NULL
END TRY
BEGIN CATCH
	DECLARE @msg nvarchar(max);
	DECLARE @severity int;
	DECLARE @state int;
	SELECT @msg = ERROR_MESSAGE(), @severity = ERROR_SEVERITY(), @state = ERROR_STATE();
	RAISERROR(@msg, @severity, @state);

	SET NOEXEC ON
END CATCH
GO
BEGIN TRY
	IF EXISTS (SELECT 1 FROM fn_listextendedproperty(N'MS_Description', 'SCHEMA', N'org', 'PROCEDURE', N'spWriteToLogTable', NULL, NULL))
EXEC sp_dropextendedproperty N'MS_Description', 'SCHEMA', N'org', 'PROCEDURE', N'spWriteToLogTable', NULL, NULL
END TRY
BEGIN CATCH
	DECLARE @msg nvarchar(max);
	DECLARE @severity int;
	DECLARE @state int;
	SELECT @msg = ERROR_MESSAGE(), @severity = ERROR_SEVERITY(), @state = ERROR_STATE();
	RAISERROR(@msg, @severity, @state);

	SET NOEXEC ON
END CATCH
GO
PRINT N'Dropping [privacy].[spProcessCleanUpList]'
GO
IF OBJECT_ID(N'[privacy].[spProcessCleanUpList]', 'P') IS NOT NULL
DROP PROCEDURE [privacy].[spProcessCleanUpList]
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Dropping [privacy].[spGetSQLCommandPartsForCurrentTable]'
GO
IF OBJECT_ID(N'[privacy].[spGetSQLCommandPartsForCurrentTable]', 'P') IS NOT NULL
DROP PROCEDURE [privacy].[spGetSQLCommandPartsForCurrentTable]
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Dropping [privacy].[spDeleteRecordsInTable]'
GO
IF OBJECT_ID(N'[privacy].[spDeleteRecordsInTable]', 'P') IS NOT NULL
DROP PROCEDURE [privacy].[spDeleteRecordsInTable]
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Dropping [privacy].[spDeleteRecordsInDetailTable]'
GO
IF OBJECT_ID(N'[privacy].[spDeleteRecordsInDetailTable]', 'P') IS NOT NULL
DROP PROCEDURE [privacy].[spDeleteRecordsInDetailTable]
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Dropping [privacy].[spDeleteJobListRecords]'
GO
IF OBJECT_ID(N'[privacy].[spDeleteJobListRecords]', 'P') IS NOT NULL
DROP PROCEDURE [privacy].[spDeleteJobListRecords]
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Dropping [privacy].[spCleanUpCallLogTable]'
GO
IF OBJECT_ID(N'[privacy].[spCleanUpCallLogTable]', 'P') IS NOT NULL
DROP PROCEDURE [privacy].[spCleanUpCallLogTable]
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Dropping [privacy].[spAddToCleanUpList]'
GO
IF OBJECT_ID(N'[privacy].[spAddToCleanUpList]', 'P') IS NOT NULL
DROP PROCEDURE [privacy].[spAddToCleanUpList]
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Dropping [privacy].[spAddEmployeeToCleanUpList]'
GO
IF OBJECT_ID(N'[privacy].[spAddEmployeeToCleanUpList]', 'P') IS NOT NULL
DROP PROCEDURE [privacy].[spAddEmployeeToCleanUpList]
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Dropping [privacy].[GenerateDeleteTriggerForOrgDocumentsTable]'
GO
IF OBJECT_ID(N'[privacy].[GenerateDeleteTriggerForOrgDocumentsTable]', 'P') IS NOT NULL
DROP PROCEDURE [privacy].[GenerateDeleteTriggerForOrgDocumentsTable]
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Dropping [org].[spWriteToLogTable]'
GO
IF OBJECT_ID(N'[org].[spWriteToLogTable]', 'P') IS NOT NULL
DROP PROCEDURE [org].[spWriteToLogTable]
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Dropping [privacy].[ufnUserCleanUpList]'
GO
IF EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[privacy].[ufnUserCleanUpList]') AND (type = 'IF' OR type = 'FN' OR type = 'TF'))
DROP FUNCTION [privacy].[ufnUserCleanUpList]
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Dropping [privacy].[ufnInvoiceListToDelete]'
GO
IF EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[privacy].[ufnInvoiceListToDelete]') AND (type = 'IF' OR type = 'FN' OR type = 'TF'))
DROP FUNCTION [privacy].[ufnInvoiceListToDelete]
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Dropping [privacy].[ufnGetBlackListRecords]'
GO
IF EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[privacy].[ufnGetBlackListRecords]') AND (type = 'IF' OR type = 'FN' OR type = 'TF'))
DROP FUNCTION [privacy].[ufnGetBlackListRecords]
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Dropping [privacy].[ufnGetAllBlackListRecordsForTable]'
GO
IF EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[privacy].[ufnGetAllBlackListRecordsForTable]') AND (type = 'IF' OR type = 'FN' OR type = 'TF'))
DROP FUNCTION [privacy].[ufnGetAllBlackListRecordsForTable]
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Dropping [privacy].[ufnEmployeeListToCleanUp]'
GO
IF EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[privacy].[ufnEmployeeListToCleanUp]') AND (type = 'IF' OR type = 'FN' OR type = 'TF'))
DROP FUNCTION [privacy].[ufnEmployeeListToCleanUp]
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Dropping [privacy].[ufnContactListToCleanUp]'
GO
IF EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[privacy].[ufnContactListToCleanUp]') AND (type = 'IF' OR type = 'FN' OR type = 'TF'))
DROP FUNCTION [privacy].[ufnContactListToCleanUp]
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Dropping [privacy].[ufnAddressListToDelete]'
GO
IF EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[privacy].[ufnAddressListToDelete]') AND (type = 'IF' OR type = 'FN' OR type = 'TF'))
DROP FUNCTION [privacy].[ufnAddressListToDelete]
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Dropping [privacy].[ufnAddressListToCleanUp]'
GO
IF EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[privacy].[ufnAddressListToCleanUp]') AND (type = 'IF' OR type = 'FN' OR type = 'TF'))
DROP FUNCTION [privacy].[ufnAddressListToCleanUp]
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Dropping [org].[fn_Split]'
GO
IF EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[org].[fn_Split]') AND (type = 'IF' OR type = 'FN' OR type = 'TF'))
DROP FUNCTION [org].[fn_Split]
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Dropping [org].[Levenshtein]'
GO
IF EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[org].[Levenshtein]') AND (type = 'IF' OR type = 'FN' OR type = 'TF'))
DROP FUNCTION [org].[Levenshtein]
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [org].[Levenshtein]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[org].[Levenshtein]') AND (type = 'IF' OR type = 'FN' OR type = 'TF'))
EXEC sp_executesql N'
-- =============================================
-- Computes and returns the Levenshtein edit distance between two strings, i.e. the
-- number of insertion, deletion, and sustitution edits required to transform one
-- string to the other, or NULL if @max is exceeded. Comparisons use the case-
-- sensitivity configured in SQL Server (case-insensitive by default).
-- http://blog.softwx.net/2014/12/optimizing-levenshtein-algorithm-in-tsql.html
-- 
-- See http://en.wikipedia.org/wiki/Levenshtein_distance
-- This is based on Sten Hjelmqvist''s "Fast, memory efficient" algorithm, described
-- at http://www.codeproject.com/Articles/13525/Fast-memory-efficient-Levenshtein-algorithm, 
-- with some additional optimizations.
-- @s - String being compared for distance.
-- @t - String being compared against other string.
-- @max - Maximum distance allowed, or NULL if no maximum is desired. Returns NULL if distance will exceed @max.
-- returns int edit distance, >= 0 representing the number of edits required to transform one string to the other.
-- =============================================
CREATE FUNCTION [org].[Levenshtein](
    @s nvarchar(4000)
  , @t nvarchar(4000)
  , @max int
)
RETURNS int
WITH SCHEMABINDING
AS
BEGIN
    DECLARE @distance int = 0 -- return variable
          , @v0 nvarchar(4000)-- running scratchpad for storing computed distances
          , @start int = 1      -- index (1 based) of first non-matching character between the two string
          , @i int, @j int      -- loop counters: i for s string and j for t string
          , @diag int          -- distance in cell diagonally above and left if we were using an m by n matrix
          , @left int          -- distance in cell to the left if we were using an m by n matrix
          , @sChar nchar      -- character at index i from s string
          , @thisJ int          -- temporary storage of @j to allow SELECT combining
          , @jOffset int      -- offset used to calculate starting value for j loop
          , @jEnd int          -- ending value for j loop (stopping point for processing a column)
          -- get input string lengths including any trailing spaces (which SQL Server would otherwise ignore)
          , @sLen int = datalength(@s) / datalength(left(left(@s, 1) + ''.'', 1))    -- length of smaller string
          , @tLen int = datalength(@t) / datalength(left(left(@t, 1) + ''.'', 1))    -- length of larger string
          , @lenDiff int      -- difference in length between the two strings
    -- if strings of different lengths, ensure shorter string is in s. This can result in a little
    -- faster speed by spending more time spinning just the inner loop during the main processing.
    IF (@sLen > @tLen) BEGIN
        SELECT @v0 = @s, @i = @sLen -- temporarily use v0 for swap
        SELECT @s = @t, @sLen = @tLen
        SELECT @t = @v0, @tLen = @i
    END
    SELECT @max = ISNULL(@max, @tLen)
         , @lenDiff = @tLen - @sLen
    IF @lenDiff > @max RETURN NULL
 
    -- suffix common to both strings can be ignored
    WHILE(@sLen > 0 AND SUBSTRING(@s, @sLen, 1) = SUBSTRING(@t, @tLen, 1))
        SELECT @sLen = @sLen - 1, @tLen = @tLen - 1
 
    IF (@sLen = 0) RETURN CASE WHEN @tLen <= @max THEN @tLen ELSE NULL END
 
    -- prefix common to both strings can be ignored
    WHILE (@start < @sLen AND SUBSTRING(@s, @start, 1) = SUBSTRING(@t, @start, 1)) 
        SELECT @start = @start + 1
    IF (@start > 1) BEGIN
        SELECT @sLen = @sLen - (@start - 1)
             , @tLen = @tLen - (@start - 1)
 
        -- if all of shorter string matches prefix and/or suffix of longer string, then
        -- edit distance is just the delete of additional characters present in longer string
        IF (@sLen <= 0) RETURN CASE WHEN @tLen <= @max THEN @tLen ELSE NULL END
 
        SELECT @s = SUBSTRING(@s, @start, @sLen)
             , @t = SUBSTRING(@t, @start, @tLen)
    END
 
    -- initialize v0 array of distances
    SELECT @v0 = '''', @j = 1
    WHILE (@j <= @tLen) BEGIN
        SELECT @v0 = @v0 + NCHAR(CASE WHEN @j > @max THEN @max ELSE @j END)
        SELECT @j = @j + 1
    END
     
    SELECT @jOffset = @max - @lenDiff
         , @i = 1
    WHILE (@i <= @sLen) BEGIN
        SELECT @distance = @i
             , @diag = @i - 1
             , @sChar = SUBSTRING(@s, @i, 1)
             -- no need to look beyond window of upper left diagonal (@i) + @max cells
             -- and the lower right diagonal (@i - @lenDiff) - @max cells
             , @j = CASE WHEN @i <= @jOffset THEN 1 ELSE @i - @jOffset END
             , @jEnd = CASE WHEN @i + @max >= @tLen THEN @tLen ELSE @i + @max END
        WHILE (@j <= @jEnd) BEGIN
            -- at this point, @distance holds the previous value (the cell above if we were using an m by n matrix)
            SELECT @left = UNICODE(SUBSTRING(@v0, @j, 1))
                 , @thisJ = @j
            SELECT @distance = 
                CASE WHEN (@sChar = SUBSTRING(@t, @j, 1)) THEN @diag                    --match, no change
                     ELSE 1 + CASE WHEN @diag < @left AND @diag < @distance THEN @diag    --substitution
                                   WHEN @left < @distance THEN @left                    -- insertion
                                   ELSE @distance                                        -- deletion
                                END    END
            SELECT @v0 = STUFF(@v0, @thisJ, 1, NCHAR(@distance))
                 , @diag = @left
                 , @j = case when (@distance > @max) AND (@thisJ = @i + @lenDiff) then @jEnd + 2 else @thisJ + 1 end
        END
        SELECT @i = CASE WHEN @j > @jEnd + 1 THEN @sLen + 1 ELSE @i + 1 END
    END
    RETURN CASE WHEN @distance <= @max THEN @distance ELSE NULL END
END
'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [privacy].[ufnGetBlackListRecords]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[privacy].[ufnGetBlackListRecords]') AND (type = 'IF' OR type = 'FN' OR type = 'TF'))
EXEC sp_executesql N'/*
Example
DECLARE @StingToSearch AS NVARCHAR(200) = LOWER(''Armaturen Voß GmbH'')
DECLARE @MaxDistance AS INT = 0

SELECT @StingToSearch
SELECT  * FROM [privacy].[ufnGetBlackListRecords](@StingToSearch,@MaxDistance)
*/
CREATE FUNCTION [privacy].[ufnGetBlackListRecords]
(
    @StingToSearch      NVARCHAR(200)
  , @MaxDistance        INT
  , @ShowAchivedRecords BIT = 0
)
RETURNS @BlackListRecords TABLE
(
    FirstName     NVARCHAR(200)
  , Lastname      NVARCHAR(200)
  , Appendix      NVARCHAR(200)
  , Street        NVARCHAR(200)
  , ZIP           NVARCHAR(200)
  , City          NVARCHAR(200)
  , Country       NVARCHAR(200)
  , EMail         NVARCHAR(200)
  , DeleteComment NVARCHAR(MAX)
  , Distance      INT
  , BlackListOID  UNIQUEIDENTIFIER
  , ID            NVARCHAR(8)
)
AS
BEGIN

    DECLARE @FilteredStringToSearch AS NVARCHAR(200);

    SET @FilteredStringToSearch = LOWER(REPLACE(@StingToSearch, '' '', ''''));

    INSERT INTO @BlackListRecords
    (
        FirstName
      , Lastname
      , Appendix
      , Street
      , ZIP
      , City
      , Country
      , EMail
      , DeleteComment
      , Distance
      , BlackListOID
      , ID
    )
    (SELECT b.Vorname
          , b.Name
          , b.Zusatz
          , b.Strasse
          , b.PLZ
          , b.Ort
          , b.Land
          , b.Email
          , b.Loeschgrund
          , org.Levenshtein(LOWER(REPLACE(ISNULL(b.Vorname, '''') + ISNULL(b.Name, '''') + ISNULL(b.Zusatz, ''''), '' '', '''')), @FilteredStringToSearch, @MaxDistance)
          , __Org_RecordOID
          , b.ID
     FROM   dbo.Blacklist b
     WHERE
            b.Archiv                                                                                                                                                                                                                                    = @ShowAchivedRecords
            AND
            (
                org.Levenshtein(LOWER(REPLACE(ISNULL(b.Vorname, '''') + ISNULL(b.Name, '''') + ISNULL(b.Zusatz, '''') + ISNULL(b.Strasse, '''') + ISNULL(b.PLZ, '''') + ISNULL(b.Ort, '''') + ISNULL(b.Land, ''''), '' '', '''')), @FilteredStringToSearch, @MaxDistance) <= @MaxDistance
                OR org.Levenshtein(LOWER(REPLACE(ISNULL(b.Strasse, '''') + ISNULL(b.PLZ, '''') + ISNULL(b.Ort, '''') + ISNULL(b.Land, '''') + ISNULL(b.Email, ''''), '' '', '''')), @FilteredStringToSearch, @MaxDistance)                                            <= @MaxDistance
                OR org.Levenshtein(LOWER(REPLACE(ISNULL(b.Email, '''') + ISNULL(b.Vorname, '''') + ISNULL(b.Name, '''') + ISNULL(b.Zusatz, ''''), '' '', '''')), @FilteredStringToSearch, @MaxDistance)                                                             <= @MaxDistance
            ));

    RETURN;
END;

'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [privacy].[ufnGetAllBlackListRecordsForTable]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[privacy].[ufnGetAllBlackListRecordsForTable]') AND (type = 'IF' OR type = 'FN' OR type = 'TF'))
EXEC sp_executesql N'
/*
Example

DECLARE @MaxDistance AS INT = 0

SELECT  * FROM [privacy].[ufnGetAllBlackListRecordsForTable](@MaxDistance) WHERE TableName = ''Adressen''
*/
CREATE FUNCTION [privacy].[ufnGetAllBlackListRecordsForTable]
(
    @MaxDistance INT
)
RETURNS TABLE
AS
RETURN
(
    SELECT  DISTINCT
            ''Adressen''                                                                                                                                                                                                                                AS TableName
          , a.__Org_RecordOID                                                                                                                                                                                                                         AS __Org_RecordOID
          , a.ID                                                                                                                                                                                                                                      AS ID
		  , LTRIM(RTRIM(a.Vorname + '' '' + a.Name + '' '' + a.Zusatz))                                                                                                                                                                                   AS Namensinformation
          --, LEFT(a.Vorname + '' '' + a.Name + '' '' + a.Zusatz + a.Strasse + a.PLZ + a.Ort + a.Land, 1000)                                                                                                                                                AS SearchValue1
          --, LEFT(a.Strasse + '' '' + a.PLZ + '' '' + a.Ort + '' '' + a.Land + a.Email, 1000)                                                                                                                                                                AS SearchValue2
          --, LEFT(a.Email + a.Vorname + '' '' + a.Name + '' '' + a.Zusatz, 1000)                                                                                                                                                                           AS SearchValue3
          --, LEFT(ISNULL(
          --                 IIF(b1.ID IS NOT NULL
          --                 , b1.FirstName + b1.Lastname + b1.Appendix + b1.Street + b1.ZIP + b1.City + b1.Country
          --                 , IIF(b2.ID IS NOT NULL, b2.FirstName + b2.Lastname + b2.Appendix + b2.Street + b2.ZIP + b2.City + b2.Country, b3.FirstName + b3.Lastname + b3.Appendix + b3.Street + b3.ZIP + b3.City + b3.Country))
          --               , ''''
          --             ), 1000)                                                                                                                                                                                                                       AS BlackListvalue1
          --, LEFT(ISNULL(IIF(b1.ID IS NOT NULL, b1.Street + b1.ZIP + b1.City + b1.Country + b1.EMail, IIF(b2.ID IS NOT NULL, b2.Street + b2.ZIP + b2.City + b2.Country + b2.EMail, b3.Street + b3.ZIP + b3.City + b3.Country + +b3.EMail)), ''''), 1000) AS BlackListvalue2
          --, LEFT(ISNULL(IIF(b1.ID IS NOT NULL, b1.EMail + b1.FirstName + b1.Lastname + b1.Appendix, IIF(b2.ID IS NOT NULL, b2.EMail + b2.FirstName + b2.Lastname + b2.Appendix, b3.EMail + b3.FirstName + b3.Lastname + b3.Appendix)), ''''), 1000)     AS BlackListvalue3
          , LEFT(ISNULL(IIF(b1.ID IS NOT NULL, b1.ID, IIF(b2.ID IS NOT NULL, b2.ID, b3.Street + b3.ID)), ''''), 8)                                                                                                                                      AS BlacklistID
		  , IIF(b1.ID IS NULL, CAST(0 AS BIT), CAST(1 AS BIT))                                                                                                                                                                                        AS BlackListFound1 
		  , IIF(b2.ID IS NULL, CAST(0 AS BIT), CAST(1 AS BIT))                                                                                                                                                                                        AS BlackListFound2 
		  , IIF(b3.ID IS NULL, CAST(0 AS BIT), CAST(1 AS BIT))                                                                                                                                                                                        AS BlackListFound3 
    FROM    dbo.Adressen AS a
        OUTER APPLY
            (
                SELECT  *
                FROM    privacy.ufnGetBlackListRecords(a.Vorname + a.Name + a.Zusatz + a.Strasse + a.PLZ + a.Ort + a.Land, @MaxDistance, 0)
            )            AS b1
        OUTER APPLY
            (
                SELECT  *
                FROM    privacy.ufnGetBlackListRecords(a.Strasse + a.PLZ + a.Ort + a.Land + a.Email, @MaxDistance, 0)
            ) AS b2
        OUTER APPLY
            (
                SELECT  *
                FROM    privacy.ufnGetBlackListRecords(a.Email + a.Vorname + a.Name + a.Zusatz, @MaxDistance, 0)
            ) AS b3
    WHERE
            b1.FirstName + b1.Lastname + b1.Appendix IS NOT NULL
            OR  b2.Street + b2.ZIP + b2.City + b2.Country IS NOT NULL
            OR  b3.EMail IS NOT NULL
    UNION ALL
    --Contacts (TABLE Ansprechpartner )
    --INSERT INTO @BlackListRecords
    SELECT  DISTINCT
            ''Ansprechpartner''                                                                                                                                                                                                                         AS TableName
          , ap.__Org_RecordOID                                                                                                                                                                                                                        AS __Org_RecordOID
          , ap.ID                                                                                                                                                                                                                                     AS ID
		  , LTRIM(RTRIM(ap.Vorname + '' '' + ap.Name + '' '' + ap.Zusatz))                                                                                                                                                                                AS Namensinformation
          --, LEFT(ap.Vorname + '' '' + ap.Name + '' '' + ap.Zusatz + a.Strasse + a.PLZ + a.Ort + a.Land, 1000)                                                                                                                                             AS SearchValue1
          --, LEFT(a.Strasse + '' '' + a.PLZ + '' '' + a.Ort + '' '' + a.Land + ap.Email, 1000)                                                                                                                                                               AS SearchValue2
          --, LEFT(ap.Email + ap.Vorname + '' '' + ap.Name + '' '' + ap.Zusatz, 1000)                                                                                                                                                                       AS SearchValue3
          --, LEFT(ISNULL(
          --                 IIF(b1.ID IS NOT NULL
          --                 , b1.FirstName + b1.Lastname + b1.Appendix + b1.Street + b1.ZIP + b1.City + b1.Country
          --                 , IIF(b2.ID IS NOT NULL, b2.FirstName + b2.Lastname + b2.Appendix + b2.Street + b2.ZIP + b2.City + b2.Country, b3.FirstName + b3.Lastname + b3.Appendix + b3.Street + b3.ZIP + b3.City + b3.Country))
          --               , ''''
          --             ), 1000)                                                                                                                                                                                                                       AS BlackListvalue1
          --, LEFT(ISNULL(IIF(b1.ID IS NOT NULL, b1.Street + b1.ZIP + b1.City + b1.Country + b1.EMail, IIF(b2.ID IS NOT NULL, b2.Street + b2.ZIP + b2.City + b2.Country + b2.EMail, b3.Street + b3.ZIP + b3.City + b3.Country + +b3.EMail)), ''''), 1000) AS BlackListvalue2
          --, LEFT(ISNULL(IIF(b1.ID IS NOT NULL, b1.EMail + b1.FirstName + b1.Lastname + b1.Appendix, IIF(b2.ID IS NOT NULL, b2.EMail + b2.FirstName + b2.Lastname + b2.Appendix, b3.EMail + b3.FirstName + b3.Lastname + b3.Appendix)), ''''), 1000)     AS BlackListvalue3
          , LEFT(ISNULL(IIF(b1.ID IS NOT NULL, b1.ID, IIF(b2.ID IS NOT NULL, b2.ID, b3.Street + b3.ID)), ''''), 8)                                                                                                                                      AS BlacklistID
		  , IIF(b1.ID IS NULL, CAST(0 AS BIT), CAST(1 AS BIT))                                                                                                                                                                                        AS BlackListFound1 
		  , IIF(b2.ID IS NULL, CAST(0 AS BIT), CAST(1 AS BIT))                                                                                                                                                                                        AS BlackListFound2 
		  , IIF(b3.ID IS NULL, CAST(0 AS BIT), CAST(1 AS BIT))                                                                                                                                                                                        AS BlackListFound3 
    FROM    dbo.Ansprechpartner AS ap
        INNER JOIN
            dbo.Adressen        AS a
                ON a.ID = ap.AdressID
        OUTER APPLY
            (
                SELECT  *
                FROM    privacy.ufnGetBlackListRecords(ap.Vorname + ap.Name + ap.Zusatz + a.Strasse + a.PLZ + a.Ort + a.Land, @MaxDistance, 0)
            )                   AS b1
        OUTER APPLY
            (
                SELECT  *
                FROM    privacy.ufnGetBlackListRecords(a.Strasse + a.PLZ + a.Ort + a.Land + ap.Email, @MaxDistance, 0)
            ) AS b2
        OUTER APPLY
            (
                SELECT  *
                FROM    privacy.ufnGetBlackListRecords(ap.Email + ap.Vorname + '' '' + ap.Name + '' '' + ap.Zusatz, @MaxDistance, 0)
            ) AS b3
    WHERE
            b1.FirstName + b1.Lastname + b1.Appendix IS NOT NULL
            OR  b2.Street + b2.ZIP + b2.City + b2.Country IS NOT NULL
            OR  b3.EMail IS NOT NULL
);

'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [privacy].[ufnInvoiceListToDelete]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[privacy].[ufnInvoiceListToDelete]') AND (type = 'IF' OR type = 'FN' OR type = 'TF'))
EXEC sp_executesql N'

-- =============================================
-- Author:		Falk Wegener
-- Create date: 2018-08-10
-- Description:	Returning a table with all invoice records wich match the criteria
-- Example: SELECT * FROM privacy.ufnInvoiceListToDelete(2)
--			
--			
-- =============================================
CREATE FUNCTION [privacy].[ufnInvoiceListToDelete]
(
    @RententionTimeInMonths4Invoices AS INT = 132   --11 years
)
RETURNS TABLE
AS
RETURN
(
    SELECT  Adressen.Name
          , Adressen.Zusatz
          , r.__Org_RecordOID
          , r.ID
          , r.Datum
          , r.Rechnungsnummer
    FROM    Adressen
        INNER JOIN
            Kundennummern
                ON Kundennummern.AdressID = Adressen.ID
        INNER JOIN
            Rechnungen AS r
                ON r.KundennummerID       = Kundennummern.ID
    WHERE   r.Datum <= DATEADD(MONTH, @RententionTimeInMonths4Invoices * (-1), GETDATE())

);

'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [privacy].[CleanUpJobList_aid]'
GO
IF OBJECT_ID(N'[privacy].[CleanUpJobList_aid]', 'U') IS NULL
CREATE TABLE [privacy].[CleanUpJobList_aid]
(
[id] [int] NOT NULL IDENTITY(1, 1),
[BatchNo] [int] NULL,
[orgAniceUser] [nvarchar] (32) COLLATE Latin1_General_CI_AS NULL,
[TableName] [nvarchar] (128) COLLATE Latin1_General_CI_AS NULL,
[TenantID] [int] NULL,
[RecordOID] [uniqueidentifier] NULL,
[RecordID] [nvarchar] (8) COLLATE Latin1_General_CI_AS NULL,
[RecordNo] [int] NULL,
[spid] [int] NOT NULL CONSTRAINT [DF__CleanUpJobList_aid__spid__7EB7AD3A] DEFAULT (@@spid),
[ActionType] [nvarchar] (16) COLLATE Latin1_General_CI_AS NOT NULL CONSTRAINT [DF_CleanUpJobList_aid_ActionType] DEFAULT (N'Clean')
)
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating primary key [PK__PrintJob__3213E83F6FBDFC03] on [privacy].[CleanUpJobList_aid]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.indexes WHERE is_primary_key= 1 AND  object_id = OBJECT_ID(N'[privacy].[CleanUpJobList_aid]'))
BEGIN
	IF NOT EXISTS (SELECT 1 FROM sys.indexes WHERE name = N'PK__PrintJob__3213E83F6FBDFC03' AND object_id = OBJECT_ID(N'[privacy].[CleanUpJobList_aid]'))
	ALTER TABLE [privacy].[CleanUpJobList_aid] ADD CONSTRAINT [PK__PrintJob__3213E83F6FBDFC03] PRIMARY KEY CLUSTERED  ([id])
END;
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [privacy].[spDeleteJobListRecords]'
GO
IF OBJECT_ID(N'[privacy].[spDeleteJobListRecords]', 'P') IS NULL
EXEC sp_executesql N'
-- =============================================
-- Author:		Falk Wegener -  the moving sql-dancer
-- Create date: 2018-03-19
-- Description:	Deleting records in temp CleanUpJobList. If  @DeleteCleanUpListForCurrentSPID is set, then Proc will only remove records for the orgAnice user and the given SPID, otherwise all records for the given orgAnice user.
-- =============================================
CREATE PROCEDURE [privacy].[spDeleteJobListRecords]
(
    @SPID AS INT,
    @DeleteCleanUpListForCurrentSPID AS bit,
    @orgAniceUser AS NVARCHAR (32)
) AS
BEGIN
IF @DeleteCleanUpListForCurrentSPID = 1
   OR   @DeleteCleanUpListForCurrentSPID = -1
BEGIN
    DELETE [privacy].CleanUpJobList_aid  WHERE   spid = @SPID AND orgAniceUser = @orgAniceUser COLLATE DATABASE_DEFAULT;
END;
ELSE
BEGIN
    DELETE [privacy].CleanUpJobList_aid WHERE   orgAniceUser = @orgAniceUser COLLATE DATABASE_DEFAULT;

END;
END;
'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [org].[spWriteToLogTable]'
GO
IF OBJECT_ID(N'[org].[spWriteToLogTable]', 'P') IS NULL
EXEC sp_executesql N'
-- =============================================
-- Author:		Falk Wegener
-- Create date: 2018-04-06
-- Description:	Add new Entry to Logtable and deletes the records from the log table, if they are older than the specified RententionPeriodInDay-Table property. if this property is not existent, then we use default value of 365.
-- =============================================
/*
--Example
DECLARE @LogID                      INT
      , @OldRecordDeletedInLogTable INT;
EXEC org.sp_WriteToLogTable @Org_Tenants = 2                                                    -- int
                          , @TimeStamp = ''2018-04-06 08:04:25''                                  -- datetime
                          , @Source = N''test''                                                   -- nvarchar(50)
                          , @Type = N''Info''                                                     -- nvarchar(20)
                          , @Message = N''Records deleted''                                       -- nvarchar(100)
                          , @Data = N''all long data deleted infos''                              -- nvarchar(max)
                          , @RecordNo = 1                                                       -- int
                          , @RecordID = N''00000001''                                             -- nvarchar(8)
                          , @Description = N''test script''                                       -- nvarchar(500)
                          , @LogID = @LogID OUTPUT                                              -- int
                          , @TableName = N''mytable''                                             -- nvarchar(128)
                          , @OldRecordDeletedInLogTable = @OldRecordDeletedInLogTable OUTPUT;   -- int



PRINT @LogID;
PRINT ''old records deleted:'' + CAST(@OldRecordDeletedInLogTable AS NVARCHAR(8));
*/
CREATE PROCEDURE [org].[spWriteToLogTable]
    @Org_Tenants                INT           = 2
  , @TimeStamp                  DATETIME      = NULL
  , @Source                     NVARCHAR(50)  = NULL
  , @Type                       NVARCHAR(20)  = NULL
  , @Message                    NVARCHAR(100) = NULL
  , @Data                       NVARCHAR(MAX) = NULL
  , @RecordNo                   INT           = NULL
  , @RecordID                         NVARCHAR(8)   = NULL
  , @Description                NVARCHAR(500) = NULL
  , @LogID                      INT           OUTPUT
  , @TableName                  NVARCHAR(128) = NULL
  , @OldRecordDeletedInLogTable INT           OUTPUT
AS
BEGIN

    DECLARE @RententionPeriod INT;
    --IF @RecordNo IS NOT NULL 
    --Begin

    INSERT INTO [org].[Log]
    (
        __Org_Tenants
      , TimeStamp
      , CreatedOn
      , CreatedBy
      , Source
      , Type
      , Message
      , Data
      , TableName
      , RecordNo
      , RecordID
      , Description
    )
    VALUES (
               @Org_Tenants
			 , @TimeStamp
			 , getdate()
			 , (
                SELECT UserName FROM dbo.__Org_Sessions WHERE   __Org_Sessions.SPID = @@SPID
                )
			 , @Source
			 , @Type
			 , @Message
			 , @Data
			 , @TableName
			 , @RecordNo
			 , @RecordID
			 , @Description
           );

    --Return last created ID to caller
    SET @LogID =
    (
        SELECT  SCOPE_IDENTITY()
    );



    --	END;
    --ELSE SET @LogID = NULL;

    --Delete records older than the configured rentention period from the table org.log
    IF EXISTS
    (
        SELECT  TOP (1)
                objtype
              , objname
              , name
              , value
        FROM    fn_listextendedproperty(NULL, ''schema'', ''org'', ''table'', ''Log'', NULL, NULL)
        WHERE   name = ''RententionTimeInDays''
    )
        SELECT  TOP (1)
            --  objtype
            --, objname
            --, name
            --, 
                @RententionPeriod = CAST([value] AS INT)
        FROM    fn_listextendedproperty(NULL, ''schema'', ''org'', ''table'', ''Log'', NULL, NULL)
        WHERE   name = ''RententionTimeInDays'';
    ELSE SET @RententionPeriod = 365;

    --delete statement
    DELETE  FROM [org].[Log] WHERE  [TimeStamp] < DATEADD(DAY, @RententionPeriod * -1, GETDATE());
    SET @OldRecordDeletedInLogTable = @@ROWCOUNT;
END;

'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [privacy].[spProcessCleanUpList]'
GO
IF OBJECT_ID(N'[privacy].[spProcessCleanUpList]', 'P') IS NULL
EXEC sp_executesql N'
-- =============================================
-- Author:		Falk Wegener
-- Create date: 2018-03-15
-- Last change: 2018-03-23
-- Description:	2018-08-23 Added check for tablename. If tableName is "Ansprechpartner" or "PGM22000_PzeMitarbeiter" then we add the column "Ausgeschieden" and set value to 1
--				2018-04-10 Added COLLATE DATABASE_DEFAULT for String joins to avoid collate errors	
--				2018-03-22 Added delete parameter for external sp calls to able to remove the records from aid table .
--						   Otherwise the external caller would need to call the delete sp too. 
--				2018-03-15 processing the records in the cleanup aid table for the given orgAnice Username or SPID
-- =============================================
CREATE PROCEDURE [privacy].[spProcessCleanUpList]
    -- Add the parameters for the stored procedure here

    @orgAniceUser                    NVARCHAR(32) = NULL
  , @SPID                            INT = NULL
  , @BatchNo                         INT = NULL OUTPUT
  , @DeleteCleanUpListForCurrentSPID BIT = 0
  , @DeleteCleanUpList               BIT = 0
AS
BEGIN
    -- SET NOCOUNT ON added to prevent extra result sets from
    -- interfering with SELECT statements.
    SET NOCOUNT ON;


    --Adding possible table PRJ_Adressen-records to joblist
    IF @DeleteCleanUpList = 1
    BEGIN
        IF EXISTS (SELECT   name FROM   sys.tables AS t WHERE   t.name = ''PRJ_ProjektAdressen'')
        BEGIN
            --DECLARE @Count AS INT = 0
            INSERT INTO privacy.[CleanUpJobList_aid]
            (
                [orgAniceUser]
              , [TableName]
              , [TenantID]
              , [RecordOID]
              , [RecordID]
                -- , [RecordNo]
              , [spid]
              , [ActionType]
            )
            --OUTPUT (SELECT COUNT(*) FROM Inserted)
            SELECT  @orgAniceUser
                  , ''PRJ_ProjektAdressen''
                  , ppa.__Org_Tenants
                  , ppa.__Org_RecordOID
                  , ppa.ID
                    --        @RecordNo ,
                  , @@SPID
                  , ''Clean''
            FROM    dbo.PRJ_ProjektAdressen    AS ppa
                INNER JOIN
                    privacy.CleanUpJobList_aid AS cujla
                        ON cujla.RecordID = ppa.AnsprechpartnerID COLLATE DATABASE_DEFAULT
            WHERE
                    cujla.orgAniceUser   = @orgAniceUser
                    AND cujla.TableName  = ''Ansprechpartner''
                    AND cujla.ActionType = ''Clean'';

            DECLARE @LogID                      INT
                  , @OldRecordDeletedInLogTable INT
                  , @TimeStamp                  AS DATETIME   = GETDATE()
                  , @Message                    NVARCHAR(100) = N''Added '' + CAST(@@ROWCOUNT AS NVARCHAR(8)) + '' records from table PRJ_Adressen to JobList'';
            EXEC [org].[spWriteToLogTable] @Org_Tenants = 2                                                     -- int
                                         , @TimeStamp = @TimeStamp                                              -- datetime
                                         , @Source = N''[privacy].[spProcessCleanUpList]''                        -- nvarchar(50)
                                         , @Type = N''Info''                                                      -- nvarchar(20)
                                         , @Message = @Message                                                  -- nvarchar(100)
                                         , @Data = N''''                                                          -- nvarchar(max)
                                         , @RecordNo = 0                                                        -- int
                                         , @RecordID = N''''                                                      -- nvarchar(8)
                                         , @Description = N''Info message for debug purpose''                     -- nvarchar(500)
                                         , @LogID = @LogID OUTPUT                                               -- int
                                         , @TableName = N''PRJ_Adressen''                                         -- nvarchar(128)
                                         , @OldRecordDeletedInLogTable = @OldRecordDeletedInLogTable OUTPUT;    -- int

        END;
    END;
    DECLARE @TableName AS NVARCHAR(128);
    DECLARE @ActionType AS NVARCHAR(16);
    --Cursor for fetching the table names from joblist, we use the joblist in order to only process the tables for the given orgAnice User
    DECLARE curTableNames CURSOR FORWARD_ONLY READ_ONLY LOCAL FOR
    SELECT  DISTINCT
            cujla.TableName
          , cujla.ActionType
    FROM    privacy.CleanUpJobList_aid AS cujla
    WHERE   cujla.orgAniceUser = @orgAniceUser; --AND cujla.[ActionType] = ''Clean''
    OPEN curTableNames;
    FETCH NEXT FROM curTableNames
    INTO @TableName
       , @ActionType;

    WHILE @@FETCH_STATUS = 0
    BEGIN
        IF @ActionType = ''Clean''
        BEGIN
            DECLARE @t TABLE
            (
                TableName  NVARCHAR(128) COLLATE DATABASE_DEFAULT
              , ColumnName NVARCHAR(128) COLLATE DATABASE_DEFAULT
            );
            DECLARE @sql AS NVARCHAR(MAX) = '''';
            INSERT INTO @t
            (
                TableName
              , ColumnName
            )
            SELECT  t.name AS TableName
                  , c.name AS ColumnName
            FROM    sys.columns AS c
                INNER JOIN
                    sys.tables  AS t
                        ON t.object_id = c.object_id
            WHERE
                    t.name               =  @TableName--''Ansprechpartner'' --''Adressen''
                    AND 
					LEFT(c.name, 6)  <> ''__Org_''
                    AND LEFT(c.name, 11) <> ''Modifiziert''
                    AND c.is_nullable    = 1
                    AND NOT EXISTS
            (
                SELECT  *
                FROM    dbo.DSGVOFeldKonfigurationen AS dfk
                WHERE
                        c.name                           = dfk.FeldName COLLATE DATABASE_DEFAULT
                        AND t.name                       = dfk.TabellenName COLLATE DATABASE_DEFAULT
                        AND
                        (
                            ISNULL(dfk.SystemFeld, 0)    = 1
                            OR
                            (
                                dfk.EigenschaftenName    = ''PrivacyDataKeepOnLeave''
                                AND dfk.EigenschaftsWert = ''True''
                            )
                        )
            );

            SET @sql =
            (
                --2018-08-23 FW Added Case When for determing, if we have the column "Ausgeschieden" or not
                SELECT      TOP (1)
                            ''UPDATE T set T.BereinigtAm=GetDate(), '' + CASE
                                                                           WHEN @TableName = ''Anpsrechpartner''
                                                                                OR  @TableName = ''PGM22000_PzeMitarbeiter'' THEN ''T.Ausgeschieden = 1,''
                                                                           ELSE ''''
                                                                       END + '' ModifiziertAm = GetDate(), ModifiziertDurch = '''''' + @orgAniceUser + '''''', '' + STUFF((
                                                                                                                                                                      SELECT    '', T.'' + QUOTENAME(t2.ColumnName) + ''= NULL''
                                                                                                                                                                      FROM      @t t2
                                                                                                                                                                      WHERE     t2.TableName = @TableName COLLATE DATABASE_DEFAULT  --''Ansprechpartner''
                                                                                                                                                                      FOR XML PATH('''')
                                                                                                                                                                  )
                                                                                                                                                                , 1
                                                                                                                                                                , 1
                                                                                                                                                                , ''''
                                                                                                                                                                 ) + '' from '' + @TableName
                            + '' AS T INNER JOIN privacy.CleanUpJobList_aid cujl ON cujl.RecordID = T.ID  COLLATE DATABASE_DEFAULT WHERE   cujl.orgAniceUser = '''''' + @orgAniceUser + ''''''  COLLATE DATABASE_DEFAULT;    -- AND cujl.spid = 70'' AS [UpdateStatement]
                FROM        @t AS t
                GROUP BY    t.TableName
                ORDER BY    1
            );

            PRINT @TableName;
            PRINT @sql;
            -- PRINT LEN(@sql);
            EXEC sp_executesql @sql;
            PRINT ''done'';
            DELETE  apm
            FROM    dbo.AnsprechpartnerMerkmale    AS apm
                INNER JOIN
                    [privacy].[CleanUpJobList_aid] cujla
                        ON cujla.RecordID = apm.AnsprechpartnerID COLLATE DATABASE_DEFAULT
            WHERE
                    cujla.TableName        = @TableName
                    AND cujla.orgAniceUser = @orgAniceUser
                    AND cujla.ActionType   = @ActionType;
        END;

        --ToDo Delete sp for L�schListen
        ELSE IF @ActionType = ''Delete'' BEGIN

PRINT ''Delete Stuff'';
        END;

        FETCH NEXT FROM curTableNames
        INTO @TableName
           , @ActionType;
    END;
    CLOSE curTableNames;

    DEALLOCATE curTableNames;
    --As wished by Xuan, we can now delete the Jolist within this sp after we have processed all records for the given user
    IF @DeleteCleanUpList = 1
    BEGIN
        --remove records in temp table for given orgAnice user, if needed
        EXEC privacy.spDeleteJobListRecords @SPID = @@SPID                                                      -- int
                                          , @DeleteCleanUpListForCurrentSPID = @DeleteCleanUpListForCurrentSPID -- bit
                                          , @orgAniceUser = @orgAniceUser;                                      -- nvarchar(32)
    END;
    SET @BatchNo = 0;
    RETURN @BatchNo;

END;



'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [privacy].[spAddToCleanUpList]'
GO
IF OBJECT_ID(N'[privacy].[spAddToCleanUpList]', 'P') IS NULL
EXEC sp_executesql N'
-- =============================================
-- Author:		Falk Wegener
-- Create date: 2018-03-15
-- Description:	Add new Entry to temp CleanUpJobList. If InstantCleanUp is set, then Proc will execute CleanUp proc after 1st Record has been added to temp Table
-- =============================================
CREATE PROCEDURE [privacy].[spAddToCleanUpList] 
	-- Add the parameters for the stored procedure here
	@InstantCleanUp BIT = 0,
	@DeleteCleanUpListForCurrentSPID bit,
	@DeleteCleanUpList bit = 1,
	@DeleteRecord BIT = 0,
	@TableName NVARCHAR(128),
	@TenantID INT,
	@RecordOID UNIQUEIDENTIFIER,
	@RecordID NVARCHAR(8),
	--@RecordNo INT,
	@BatchNo INT = NULL OUTPUT,
	@orgAniceUser NVARCHAR(32) = NULL--,
--	@TransactionID INT OUTPUT
, @AddressID as nvarchar(8) = NULL
, @CleanUpAllAddressContacts as bit = 0 
, @ActionType AS NVARCHAR(16) = ''Clean''

AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;
PRINT @DeleteCleanUpList;
--Only if list shall be Cleaned
If @DeleteCleanUpList = 1 
BEGIN
--remove records in temp table for given orgAnice user, if needed
EXEC privacy.spDeleteJobListRecords @SPID = @@SPID                                                        -- int
                                , @DeleteCleanUpListForCurrentSPID = @DeleteCleanUpListForCurrentSPID   -- bit
                                , @orgAniceUser = @orgAniceUser;                                        -- nvarchar(32)
END;


IF @CleanUpAllAddressContacts = 0
BEGIN
    INSERT INTO privacy.[CleanUpJobList_aid]
    (
        [orgAniceUser]
      , [TableName]
      , [TenantID]
      , [RecordOID]
      , [RecordID]
        -- , [RecordNo]
      , [spid]
      , [ActionType]
    )
    VALUES (
               @orgAniceUser, @TableName, @TenantID, @RecordOID, @RecordID  --        @RecordNo ,
             , @@SPID, @ActionType
           );
END;
ELSE
BEGIN
    INSERT INTO privacy.[CleanUpJobList_aid]
    (
        [orgAniceUser]
      , [TableName]
      , [TenantID]
      , [RecordOID]
      , [RecordID]
        -- , [RecordNo]
      , [spid]
      , [ActionType]
    )
    SELECT  @orgAniceUser
          , @TableName
          , @TenantID
          , ap.__Org_RecordOID
          , ap.ID
            --        @RecordNo ,
          , @@SPID
          , @ActionType
    FROM    dbo.Ansprechpartner AS ap
    WHERE   ap.AdressID = @AddressID;
END;


--If option is set, then the private data will be purged from the records which have been added to the job table. 
IF @InstantCleanUp = 1 OR @InstantCleanUp = -1
    BEGIN
		--decide, if current SPID records should be used or all records for the orgAnice user name.
		-- we don''t use the delete new parameter. We delete the records in this sp.  
		IF @DeleteCleanUpListForCurrentSPID = 0
		BEGIN
			EXEC [privacy].spProcessCleanUpList @orgAniceUser = @orgAniceUser   -- nvarchar(32)
												, @SPID = 0                       -- int
												, @BatchNo = @BatchNo OUTPUT
												, @DeleteCleanUpListForCurrentSPID = DEFAULT
												, @DeleteCleanUpList = DEFAULT;   -- int
		END;

		ELSE
		BEGIN
			EXEC [privacy].spProcessCleanUpList @orgAniceUser = @orgAniceUser   -- nvarchar(32)
												, @SPID = @@SPID                  -- int
												, @BatchNo = @BatchNo OUTPUT      -- int   
												, @DeleteCleanUpListForCurrentSPID = DEFAULT
												, @DeleteCleanUpList = DEFAULT;   -- int   
		END;

PRINT ''    Private data purge complete'';
		IF @DeleteRecord = 1 OR @DeleteRecord = -1
    BEGIN
--        EXEC [doc].[sp_Create] @@SPID, @PrintUser,@BatchNo OUTPUT;
PRINT ''    Delete record and all related records'';
		
    END;
    END;

									 
--Only if list shall be Cleaned
If @DeleteCleanUpList = 1 
BEGIN
--remove newly added record if needed
EXEC privacy.spDeleteJobListRecords @SPID = @@SPID                                                        -- int
                                , @DeleteCleanUpListForCurrentSPID = @DeleteCleanUpListForCurrentSPID   -- bit
                                , @orgAniceUser = @orgAniceUser;                                        -- nvarchar(32)

END;

	SET @BatchNo = 0;
RETURN @BatchNo;




END;



'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [privacy].[ufnUserCleanUpList]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[privacy].[ufnUserCleanUpList]') AND (type = 'IF' OR type = 'FN' OR type = 'TF'))
EXEC sp_executesql N'

-- =============================================
-- Author:		Falk Wegener orgAnice DevLeader
-- Create date: 2018-03-15
-- Description:	Returning a table with the current job list for a orgAnice User 
--
-- Example: SELECT * FROM [privacy].[ufnUserCleanUpList](''Admin'',null)
-- =============================================
CREATE FUNCTION [privacy].[ufnUserCleanUpList]
(
    @orgAniceUser AS NVARCHAR(31) = ''''
  , @SPID AS         INT          = NULL    -- if NULL then we use the current session spid
)
RETURNS TABLE
AS
RETURN
(
    SELECT  *
    FROM    privacy.CleanUpJobList_aid AS cujla
    WHERE
            (
                @SPID IS NULL
                AND cujla.orgAniceUser = @orgAniceUser
            )
            OR
            (
                @orgAniceUser          = ''''
                AND cujla.spid              = @SPID
            
            )            OR
            (
                cujla.orgAniceUser     = @orgAniceUser
                AND @SPID              = @@SPID
            )
);




'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [privacy].[spGetSQLCommandPartsForCurrentTable]'
GO
IF OBJECT_ID(N'[privacy].[spGetSQLCommandPartsForCurrentTable]', 'P') IS NULL
EXEC sp_executesql N'-- =============================================
-- Author			: Falk Wegener
-- Create date		: 2018-03-29
-- last change date	: 2018-08-29
-- Description:	Outsourced code for generating the strings for the sql command in the sp [privacy].[spDeleteRecordsInTable]
-- =============================================

CREATE PROCEDURE [privacy].[spGetSQLCommandPartsForCurrentTable]
(
    @TableAliasName AS      NVARCHAR(200)
  , @ColumnList             NVARCHAR(4000) = '''' OUTPUT
  , @TableVariableStatement NVARCHAR(4000) = '''' OUTPUT
  , @OutputClauseStatement  NVARCHAR(4000) = '''' OUTPUT
  , @UnPivotStatement       NVARCHAR(4000) = '''' OUTPUT
)
AS
BEGIN
    DECLARE @TableName NVARCHAR(128) = '''';
    --DECLARE @ColumnList NVARCHAR(4000) = '''';
    --DECLARE @TableVariableStatement NVARCHAR(4000) = '''';
    --DECLARE @OutputClauseStatement NVARCHAR(4000) = '''';

    --SET @TableAliasName = N''Adressen_1'';
    --Remove the AliasSuffix
    SET @TableName = LEFT(@TableAliasName, LEN(@TableAliasName) - 2);
    PRINT @TableName;
    SELECT  @TableVariableStatement = N''DECLARE @DeletedRecords as Table ([OID]			 UNIQUEIDENTIFIER, '' + STUFF((
                                                                                                                         SELECT '', '' + QUOTENAME(c.name) + '' UNIQUEIDENTIFIER ''
                                                                                                                         FROM   sys.columns AS c
                                                                                                                         WHERE
                                                                                                                                c.object_id          = t.object_id
                                                                                                                                AND c.system_type_id = 36
                                                                                                                                AND c.name           <> ''__Org_RecordOID''
                                                                                                                                AND LEFT(t.name, 6)  <> ''__Org_''
                                                                                                                         FOR XML PATH(''''), TYPE
                                                                                                                     ).value(''.'', ''NVARCHAR(MAX)'')
                                                                                                                   , 1
                                                                                                                   , 2
                                                                                                                   , ''''
                                                                                                                    ) + N'')''
          , @ColumnList             = STUFF((
                                                SELECT  '', '' + QUOTENAME(c.name)
                                                FROM    sys.columns AS c
                                                WHERE
                                                        c.object_id          = t.object_id
                                                        AND c.system_type_id = 36
                                                        AND c.name           <> ''__Org_RecordOID''
                                                        AND LEFT(t.name, 6)  <> ''__Org_''
                                                FOR XML PATH(''''), TYPE
                                            ).value(''.'', ''NVARCHAR(MAX)'')
                                          , 1
                                          , 2
                                          , ''''
                                           )
          , @OutputClauseStatement  = N'' OUTPUT Deleted.__Org_RecordOID,  '' + STUFF((
                                                                                        SELECT      '', '' + ''Deleted.'' + QUOTENAME(c.name)
                                                                                        FROM        sys.columns AS c
                                                                                        WHERE
                                                                                                    c.object_id          = t.object_id
                                                                                                    AND c.system_type_id = 36
                                                                                                    AND c.name           <> ''__Org_RecordOID''
                                                                                                    AND LEFT(t.name, 6)  <> ''__Org_''
                                                                                        FOR XML PATH(''''), TYPE
                                                                                    ).value(''.'', ''NVARCHAR(MAX)'')
                                                                                  , 1
                                                                                  , 2
                                                                                  , ''''
                                                                                   ) + N'' INTO @DeletedRecords''
    FROM    sys.tables                                                                    AS t
        CROSS APPLY
            (SELECT QUOTENAME(OBJECT_SCHEMA_NAME(t.object_id)) + ''.'' + QUOTENAME(t.name)) o(ObjectName)
    WHERE
            t.name = @TableName
            AND EXISTS
    (
        SELECT  1
        FROM    sys.columns AS c
        WHERE
            --c.Name = ''LastUpdate'' 
                c.system_type_id        = 36
                AND c.name              <> ''__Org_RecordOID''
                AND LEFT(t.name, 6)     <> ''__Org_''
                AND LEFT(ObjectName, 5) = ''[dbo]''
                AND c.object_id         = t.object_id
    );

    SET @UnPivotStatement
        = '' DECLARE @DeletedDocOIDs TABLE
(
    TName      NVARCHAR(128)
  , OID         UNIQUEIDENTIFIER
  , ColumnName NVARCHAR(128)
  , DocOID     UNIQUEIDENTIFIER
);

INSERT INTO @DeletedDocOIDs
(
    TName
  , OID
  , ColumnName
  , DocOID
)
(
SELECT  TName
      , OID
      , ColumnName
      , DocOID
FROM
        (
            SELECT  '''''' + @TableName + '''''' AS TName
                  , t.OID
                  , '' + @ColumnList + ''
            FROM     @DeletedRecords t
) a  
        
    UNPIVOT
    (
        DocOID
        FOR ColumnName IN
        ('' + @ColumnList
          + '')
    ) u
	);

INSERT INTO privacy.__Org_Documents_ExternalFileNamesFromDeletedRecords
    (
        DocOid
      , DocLink
      , psDeleteCommand
      , AngelegtAm
      , AngelegtDurch
      , ModifiziertAm
      , ModifiziertDurch
      , DeletedOn
      , DeletedBy
    )

	SELECT od.DocOid
          , od.DocLink 
		  , ''''Remove-Item '''' + od.DocLink AS psDeleteCommand
          , GETDATE()
          , (
                SELECT  UserName FROM dbo.__Org_Sessions WHERE   __Org_Sessions.SPID = @@SPID
            )
          , GETDATE()
          , (
                SELECT  UserName FROM dbo.__Org_Sessions WHERE   __Org_Sessions.SPID = @@SPID
            )
          , GETDATE()
          , (
                SELECT  UserName FROM dbo.__Org_Sessions WHERE   __Org_Sessions.SPID = @@SPID
            )
		  FROM @DeletedDocOIDs AS ddod
		  
INNER JOIN dbo.__Org_Documents AS od ON od.DocOid = ddod.DocOID
            WHERE od.DocType          = 1
            AND LEN(od.DocLink) > 6
            AND LEFT(od.DocLink, 4)NOT IN
    ( ''''http'''', ''''ftp:'''' );
	
--Finally delete records in __Org_Document table
DELETE  od  FROM dbo.__Org_Documents od INNER JOIN  @DeletedDocOIDs AS t ON t.DocOID = od.DocOid;
--Log Deleted records in LogTable

DECLARE @DocOIDs NVARCHAR(4000);
SET @DocOIDs = N'''''''';
IF (SELECT COUNT(*) FROM @DeletedDocOIDs) > 0
BEGIN
SELECT  @DocOIDs = @DocOIDs + N'''';'''' + CAST([DocOID] AS NVARCHAR(40)) FROM @DeletedDocOIDs;

DECLARE @Timestamp DATETIME = GETDATE();
DECLARE @Message NVARCHAR(100) = '''''''';
DECLARE @Data NVARCHAR(MAX) = '''''''';
SET @Message = ISNULL(LEFT(CAST((SELECT COUNT(*) FROM @DeletedDocOIDs) AS NVARCHAR(8)) + '''' Records in table dbo.__Org_Documents deleted.'''', 100), ''''''''); -- nvarchar(100)
SET @Data = N''''All related records with the following DocOIDs have been deleted (Max 4000 characters logged)'''' + CHAR(13) + CHAR(10) + @DocOIDs;
EXEC org.spWriteToLogTable @Org_Tenants = 2                                 -- int
                         , @TimeStamp = @Timestamp                          -- datetime
                         , @Source = N''''[privacy].[spDeleteRecordsInTable]''''  -- nvarchar(50)
                         , @Type = N''''Info''''                                  -- nvarchar(20)
                         , @Message = @Message
                         , @Data = @Data
                         , @RecordNo = NULL                                 -- int
                         , @RecordID = NULL                                 -- nvarchar(8)
                         , @Description = N''''Delete info for CleanUp list''''   -- nvarchar(500)
                         , @LogID = NULL                                    -- int
                         , @TableName = ''''__Org_Documents''''                -- nvarchar(128)
                         , @OldRecordDeletedInLogTable = NULL;              -- int;
END;'';
END;

'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [org].[fn_Split]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[org].[fn_Split]') AND (type = 'IF' OR type = 'FN' OR type = 'TF'))
EXEC sp_executesql N'CREATE function [org].[fn_Split] ( 
	@string nvarchar(4000), 
	@delimiter nvarchar(5) 
) 

returns @splittedValues table 
( 
	OccurenceId smallint identity(1, 1), 
	SplitValue nvarchar(4000) NULL
) 

as 
begin 
	declare @splitLength int;

	while len(@string) > 0 begin 
		select @splitLength = (case charindex(@delimiter, @string)
			when 0 then len(@string) else charindex(@delimiter, @string) -1 end) ;

		insert into @splittedValues select substring(@string, 1, @splitLength); 

		select @string = (case (len(@string) - @splitLength) when 0 then '''' else right(@string, len(@string) - @splitLength - 1) end);
	end ;

	return ;

end;'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [privacy].[spDeleteRecordsInTable]'
GO
IF OBJECT_ID(N'[privacy].[spDeleteRecordsInTable]', 'P') IS NULL
EXEC sp_executesql N'



-- =============================================
-- Author			: Falk Wegener
-- Create date		: 2018-03-26
-- last change date	: 2018-08-29
-- Description:	Deleting records from a detail tables based on mastertable FK key. All detail tables alias names must be supplied as comma separated list in ascending order (bottom up)
-- =============================================

CREATE PROCEDURE [privacy].[spDeleteRecordsInTable]
    @TopTableName AS           NVARCHAR(128)
  , @TopTableAliasName AS      NVARCHAR(128)
  , @DeleteTableAliasNames AS  NVARCHAR(4000)
  , @TableJoinStatement AS     NVARCHAR(4000)
  , @TopTableIDColumnName AS   NVARCHAR(128)
  , @TopTableIDColumnValues AS NVARCHAR(4000)
  , @RecordsDeleted            INT            OUTPUT
  , @ReturnText                NVARCHAR(4000) OUTPUT
AS
BEGIN
    -- SET NOCOUNT ON added to prevent extra result sets from
    -- interfering with SELECT statements.
    --SET NOCOUNT OFF;
    DECLARE @sqlCmd AS NVARCHAR(MAX) = '''';
    DECLARE @TableAliasName AS NVARCHAR(200);
    --DECLARE @ExternalDocLinks TABLE ( DocOID  UNIQUEIDENTIFIER,LinkName  NVARCHAR(255));

    DECLARE @LogID     AS INT
          , @Timestamp DATETIME
          , @Message   NVARCHAR(100)
          , @Data      NVARCHAR(MAX)
          , @RC        AS INT;

    DECLARE @rowcount TABLE
    (
        Value INT
    );


    --2018-0828 FW added new table var for fetching the GUIDs in order to log the external filenames for later removement procedure
    DECLARE @DeletedDocOIDs TABLE
    (
        TName      NVARCHAR(128)
      , ID         NVARCHAR(8)
      , ColumnName NVARCHAR(128)
      , DocOID     UNIQUEIDENTIFIER
    );
    --2018-08-29 FWnew variables for external document handling
    DECLARE @ColumnList NVARCHAR(4000);
    DECLARE @TableVariableStatement NVARCHAR(4000);
    DECLARE @OutputClauseStatement NVARCHAR(4000);
    DECLARE @UnPivotStatement NVARCHAR(4000);

    SET @RecordsDeleted = 0;
    BEGIN TRY

        -- start a transaction
        BEGIN TRAN;

        DECLARE cur CURSOR FAST_FORWARD READ_ONLY FOR
        SELECT  SplitValue FROM org.fn_Split(LTRIM(RTRIM(@DeleteTableAliasNames)), '','');
        OPEN cur;
        FETCH NEXT FROM cur
        INTO @TableAliasName;
        WHILE @@FETCH_STATUS = 0
        BEGIN
            DELETE  FROM @rowcount;
            --parse delete statement like
            --DELETE t1 FROM dbo.Aktivitaeten t1 
            --	INNER JOIN  dbo.Vorgaenge AS t2 ON t2.ID = t1.VorgangID
            --	WHERE t2.AdressID =''0000000

            --2018-08-29 FW Added OUTPUT clause and collecting the orgAnice document columns
            SET @ColumnList = N'''';
            SET @TableVariableStatement = N'''';
            SET @OutputClauseStatement = N'''';
            SET @UnPivotStatement = N'''';

            EXEC privacy.spGetSQLCommandPartsForCurrentTable @TableAliasName
                                                           , @ColumnList OUTPUT
                                                           , @TableVariableStatement OUTPUT
                                                           , @OutputClauseStatement OUTPUT
                                                           , @UnPivotStatement OUTPUT;

            -- 2018-08-29 New table variable
            IF @ColumnList <> '''' SET @sqlCmd = @TableVariableStatement + '';''; ELSE SET @sqlCmd = '''';
            SET @sqlCmd += ''DELETE '' + LTRIM(RTRIM(@TableAliasName));
            IF @ColumnList <> '''' SET @sqlCmd += @OutputClauseStatement;
            SET @sqlCmd += ''  FROM '' + LTRIM(RTRIM(@TopTableName)) + '' as '' + LTRIM(RTRIM(@TopTableAliasName));
            --''+='' add String to string
            SET @sqlCmd += '' '' + LTRIM(RTRIM(@TableJoinStatement));
            --SET @sqlCmd += '' Where '' + @TopTableAliasName + ''.'' + @TopTableIDColumnName + '' in ('' + @TopTableIDColumnValues + '')'';
            SET @sqlCmd += '' Where EXISTS (SELECT * FROM org.fn_Split(N'''''' + LTRIM(RTRIM(@TopTableIDColumnValues)) + '''''','''','''') AS fs WHERE fs.SplitValue = '' + LTRIM(RTRIM(@TopTableAliasName)) + ''.'' + LTRIM(RTRIM(@TopTableIDColumnName)) + '')'';
            SET @sqlCmd += '' SELECT RC = @@ROWCOUNT;'';
            IF @ColumnList <> '''' SET @sqlCmd += @UnPivotStatement;
            --PRINT LEFT(@sqlCmd,4000);
            --PRINT Right(@sqlCmd,4000);
            --SELECT  @TableAliasName, @sqlCmd;
            --EXEC @RC = sys.sp_executesql @sqlCmd;
            INSERT INTO @rowcount EXEC (@sqlCmd);
            SET @RC =
            (
                SELECT  Value FROM  @rowcount
            );

            SET @RecordsDeleted += @RC;
            SET @Timestamp = GETDATE();
            SET @Message = ISNULL(LEFT(CAST(@RC AS NVARCHAR(8)) + '' Records in '' + @TableAliasName + '' deleted.'', 100), ''''); -- nvarchar(100)
            SET @Data = N''All related records for the following master IDs have been deleted in table '' + @TableAliasName + N'': '' + CHAR(13) + CHAR(10) + @TopTableIDColumnValues;
            EXEC org.spWriteToLogTable @Org_Tenants = 2                                 -- int
                                     , @TimeStamp = @Timestamp                          -- datetime
                                     , @Source = N''[privacy].[spDeleteRecordsInTable]''  -- nvarchar(50)
                                     , @Type = N''Info''                                  -- nvarchar(20)
                                     , @Message = @Message
                                     , @Data = @Data
                                     , @RecordNo = NULL                                 -- int
                                     , @RecordID = NULL                                 -- nvarchar(8)
                                     , @Description = N''Delete info for CleanUp list''   -- nvarchar(500)
                                     , @LogID = NULL                                    -- int
                                     , @TableName = @TableAliasName                     -- nvarchar(128)
                                     , @OldRecordDeletedInLogTable = NULL;              -- int


            --PRINT ''-- '' + @TableAliasName;

            FETCH NEXT FROM cur
            INTO @TableAliasName;

        END;

        CLOSE cur;
        DEALLOCATE cur;

        --finally delete records from master table
        DELETE  FROM @rowcount;

        --2018-08-29 FW adding document removal methos to master table, too.
        SET @ColumnList = N'''';
        SET @TableVariableStatement = N'''';
        SET @OutputClauseStatement = N'''';
        SET @UnPivotStatement = N'''';

        EXEC privacy.spGetSQLCommandPartsForCurrentTable @TopTableAliasName
                                                       , @ColumnList OUTPUT
                                                       , @TableVariableStatement OUTPUT
                                                       , @OutputClauseStatement OUTPUT
                                                       , @UnPivotStatement OUTPUT;

        --DELETE FROM Adressen Where EXISTS (SELECT * FROM org.fn_Split(N''00148456,00178215,00102183,00151014'','','') AS fs WHERE fs.SplitValue = Adressen.ID) SELECT RC = @@ROWCOUNT
        IF @ColumnList <> '''' SET @sqlCmd = @TableVariableStatement + '';''; ELSE SET @sqlCmd = '''';
        SET @sqlCmd += ''DELETE FROM '' + LTRIM(RTRIM(@TopTableName));
        IF @ColumnList <> '''' SET @sqlCmd += @OutputClauseStatement;
        SET @sqlCmd += '' Where EXISTS (SELECT * FROM org.fn_Split(N'''''' + LTRIM(RTRIM(@TopTableIDColumnValues)) + '''''','''','''') AS fs WHERE fs.SplitValue = '' + LTRIM(RTRIM(@TopTableName)) + ''.'' + LTRIM(RTRIM(@TopTableIDColumnName)) + '')'';

        SET @sqlCmd += '' SELECT RC = @@ROWCOUNT'';
        IF @ColumnList <> '''' SET @sqlCmd += @UnPivotStatement;
        PRINT @sqlCmd;

        INSERT INTO @rowcount EXEC (@sqlCmd);
        SET @RC =
        (
            SELECT  Value FROM  @rowcount
        );

        SET @RecordsDeleted += @RC;
        SET @Timestamp = GETDATE();
        SET @Message = ISNULL(LEFT(CAST(@RC AS NVARCHAR(8)) + '' Records in Master table '' + @TopTableName + '' deleted.'', 100), ''''); -- nvarchar(100)
        SET @Data = N''All related records for the following IDs have been deleted in table '' + @TopTableName + N'': '' + CHAR(13) + CHAR(10) + @TopTableIDColumnValues;
        EXEC org.spWriteToLogTable @Org_Tenants = 2                                 -- int
                                 , @TimeStamp = @Timestamp                          -- datetime
                                 , @Source = N''[privacy].[spDeleteRecordsInTable]''  -- nvarchar(50)
                                 , @Type = N''Info''                                  -- nvarchar(20)
                                 , @Message = @Message
                                 , @Data = @Data
                                 , @RecordNo = NULL                                 -- int
                                 , @RecordID = NULL                                 -- nvarchar(8)
                                 , @Description = N''Delete info for CleanUp list''   -- nvarchar(500)
                                 , @LogID = NULL                                    -- int
                                 , @TableName = @TableAliasName                     -- nvarchar(128)
                                 , @OldRecordDeletedInLogTable = NULL;              -- int

        IF @RecordsDeleted IS NULL SET @RecordsDeleted = 0;

        -- commit the transaction
        COMMIT TRAN;
		SET @ReturnText = ''OK''

    END TRY
    BEGIN CATCH
        -- we''re in here which means an error occurred!

        -- if all transactions have not already been rolled back or are not otherwise dead roll it back
        IF (XACT_STATE() != 0) ROLLBACK TRAN;

        -- display and log the error message    
        SET @ReturnText = ''ERROR: ''
        (
            SELECT  ERROR_MESSAGE() AS [error_message]
        );
        SET @Timestamp = GETDATE();
        SET @Message = N''Error while executing [privacy].[spDeleteRecordsInTable]'';
        SET @Data = @ReturnText;
        EXEC org.spWriteToLogTable @Org_Tenants = 2                                 -- int
                                 , @TimeStamp = @Timestamp                          -- datetime
                                 , @Source = N''[privacy].[spDeleteRecordsInTable]''  -- nvarchar(50)
                                 , @Type = N''Error''                                 -- nvarchar(20)
                                 , @Message = @Message
                                 , @Data = @Data
                                 , @RecordNo = NULL                                 -- int
                                 , @RecordID = NULL                                 -- nvarchar(8)
                                 , @Description = N''Error catched within try catch'' -- nvarchar(500)
                                 , @LogID = NULL                                    -- int
                                 , @TableName = @TableAliasName                     -- nvarchar(128)
                                 , @OldRecordDeletedInLogTable = NULL;              -- int
    END CATCH;
    RETURN @RecordsDeleted;
    RETURN @ReturnText;
END;




'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [privacy].[ufnAddressListToCleanUp]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[privacy].[ufnAddressListToCleanUp]') AND (type = 'IF' OR type = 'FN' OR type = 'TF'))
EXEC sp_executesql N'

-- =============================================
-- Author:		Falk Wegener
-- Create date: 2018-03-20
-- Last change: 2018-08-08 Removed all parameters since not used/wanted anymore.
--				2018-03-22 Disabled tenant filtering we return now the tenant column instead
--
-- Description:	Returning a table with all contact records wich match the criteria
--				PDP is the abrevation used for PrivateDataProtection
--				If no permission is set (Feature not active), then You can just use the default value for the related parameter

-- Example: SELECT  COUNT(ucltcu.__Org_RecordNo) from privacy.ufnAddressListToCleanUp() AS ucltcu WHERE ucltcu.__Org_RecordNo = 66
--			''Default parameter value
--			SELECT * from privacy.ufnAddressListToCleanUp()
-- =============================================
CREATE FUNCTION [privacy].[ufnAddressListToCleanUp]
()
RETURNS @result TABLE
(
    [__Org_RecordOID] [UNIQUEIDENTIFIER] NOT NULL
  , [__Org_Tenants]   INT                NOT NULL
  , [ID]              [NVARCHAR](8)      NOT NULL
  , [An]              [NVARCHAR](100)    NULL
  , [Titel]           [NVARCHAR](160)    NULL
  , [Vorname]         [NVARCHAR](360)    NULL
  , [Name]            [NVARCHAR](360)    NULL
  , [Zusatz]          [NVARCHAR](360)    NULL
  , [Strasse]         [NVARCHAR](360)    NULL
  , [PLZ]             [NVARCHAR](100)    NULL
  , [Ort]             [NVARCHAR](230)    NULL
  , [Land]            [NVARCHAR](30)     NULL
  , [TableName]       [NVARCHAR](128)    NOT NULL
)
AS
BEGIN
    INSERT INTO @result
    SELECT  DISTINCT
            a.__Org_RecordOID
          , a.__Org_Tenants
          , LEFT(a.Id, 8)
          , LEFT(a.An, 100)
          , LEFT(a.Titel, 160)
          , LEFT(a.Vorname, 360)
          , LEFT(a.Name, 360)
          , LEFT(a.Zusatz, 360)
          , LEFT(a.Strasse, 360)
          , LEFT(a.PLZ, 100)
          , LEFT(a.Ort, 230)
          , LEFT(a.Land, 30)
          , ''Adressen'' AS TableName
    FROM    dbo.Adressen        AS a
        INNER JOIN
            dbo.Ansprechpartner AS ap
                ON ap.AdressID = a.Id
    WHERE
            a.Sperrvermerk                   = ''Firma erloschen''
            AND
            (
                ap.Ausgeschieden             = 0
                OR
                (
                    ap.BereinigtAm IS NULL --column never used
                    OR  YEAR(ap.BereinigtAm) = 1899
                )
            );
    RETURN;
END;






'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [privacy].[spCleanUpCallLogTable]'
GO
IF OBJECT_ID(N'[privacy].[spCleanUpCallLogTable]', 'P') IS NULL
EXEC sp_executesql N'

-- =============================================
-- Author:		Falk Wegener
-- Create date: 2018-03-20
-- Description:	CleanUp of the table ORGPHONE_CallLog. 
-- =============================================
CREATE PROCEDURE [privacy].[spCleanUpCallLogTable]
    @RententionTimeInDays INT = 30
	,@RecordsDeleted INT OUTPUT
AS
BEGIN
    -- SET NOCOUNT ON added to prevent extra result sets from
    -- interfering with SELECT statements.
    --SET NOCOUNT OFF;

	DELETE FROM dbo.ORGPHONE_CallLog
	WHERE DATEADD(DAY, @RententionTimeInDays,BeginTime) <= GETDATE()

	SET @RecordsDeleted= @@ROWCOUNT
	RETURN @RecordsDeleted
	END;



'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [privacy].[ufnContactListToCleanUp]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[privacy].[ufnContactListToCleanUp]') AND (type = 'IF' OR type = 'FN' OR type = 'TF'))
EXEC sp_executesql N'
-- =============================================
-- Author:		Falk Wegener
-- Create date: 2018-03-14
-- Last change: 2018-08-22
-- Description:	Returning a table with all contact records wich match the criteria
--				PDP is the abrevation used for PrivateDataProtection
--				If no permission is set (Feature not active), then You can just use the default value for the related parameter

-- Example: SELECT  COUNT(ucltcu.__Org_RecordNo) from privacy.ufnContactListToCleanUp() AS ucltcu WHERE ucltcu.__Org_RecordNo = 66
--			''Default parameter value
--			SELECT * from privacy.ufnContactListToCleanUp()
-- =============================================
CREATE FUNCTION [privacy].[ufnContactListToCleanUp]
()
RETURNS TABLE
AS
RETURN
(
    SELECT  ap.__Org_RecordOID
          , ap.Vorname
          , ap.Name
          , ap.ID
          , ''Ansprechpartner'' AS TableName
          , a.__Org_RecordOID AS Adressen___Org_RecordOID
          , a.[ID]            AS Adressen_ID
          , a.[Name]          AS Adressen_Name
          , a.[Zusatz]        AS Adressen_Zusatz
    FROM    dbo.Ansprechpartner AS ap
        INNER JOIN
            dbo.Adressen        AS a
                ON a.ID = ap.AdressID
    WHERE
            (
                ap.Ausgeschieden             = 1
                AND
                (
                    ap.BereinigtAm IS NULL --column never used
                    OR  YEAR(ap.BereinigtAm) = 1899 --column used and date removed again by orgAnice application
                )
                AND
                (
                    ap.PrivateDatenBehalten  = 0
                    OR  ap.PrivateDatenBehalten IS NULL
                )
            )
            OR
            (
                ap.Ausgeschieden             = 1
                AND
                (
                    ap.BereinigtAm IS NULL --column never used
                    OR  YEAR(ap.BereinigtAm) = 1899 --column used and date removed again by orgAnice application
                )
                AND
                (
                    ap.PrivateDatenBehalten  = 0
                    OR  ap.PrivateDatenBehalten IS NULL
                )
            )
	--2018-09-20 FW Union deaktivated, because we have only one configuration set per table and we don''t support multiple cleanup settings for each addres type
    --UNION ALL
    --SELECT  a.__Org_RecordOID
    --      , a.Vorname
    --      , a.Name
    --      , a.ID
    --      , ''Adressen''        AS TableName
    --      , a.__Org_RecordOID AS Adressen___Org_RecordOID
    --      , a.[ID]            AS Adressen_ID
    --      , a.[Name]          AS Adressen_Name
    --      , a.[Zusatz]        AS Adressen_Zusatz
    --FROM    dbo.Adressen AS a
    --WHERE
    --        (
    --            a.BereinigtAm IS NULL --column never used
    --            OR  YEAR(a.BereinigtAm) = 1899 --column used and date removed again by orgAnice application
    --        )
    --        AND
    --        (
    --            a.PrivateDatenBehalten  = 0
    --            OR  a.PrivateDatenBehalten IS NULL
    --        )
    --        AND (a.Rechtsperson            = ''Nat�rliche Person'')
    --        AND (a.Sperrvermerk         = ''Firma erloschen'')
);




'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [privacy].[spAddEmployeeToCleanUpList]'
GO
IF OBJECT_ID(N'[privacy].[spAddEmployeeToCleanUpList]', 'P') IS NULL
EXEC sp_executesql N'

-- =============================================
-- Author:		Falk Wegener
-- Create date: 2018-03-15
-- Description:	Add new Entry to temp CleanUpJobList. If InstantCleanUp is set, then Proc will execute CleanUp proc after 1st Record has been added to temp Table
-- =============================================
CREATE PROCEDURE [privacy].[spAddEmployeeToCleanUpList] 
	-- Add the parameters for the stored procedure here
	@InstantCleanUp BIT = 0,
	@DeleteCleanUpList BIT = 1,
	@DeleteRecord BIT = 0,
	@TableName NVARCHAR(128),
	@TenantID INT,
	@RecordOID UNIQUEIDENTIFIER,
	@RecordID NVARCHAR(8),
	@BatchNo INT = NULL OUTPUT,
	@orgAniceUser NVARCHAR(32)  = NULL--,
--, @ActionType AS NVARCHAR(16) = ''Clean''

AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;
--PRINT @DeleteCleanUpList;

EXEC privacy.spAddToCleanUpList @InstantCleanUp = @InstantCleanUp                  -- bit
                              , @DeleteCleanUpListForCurrentSPID = 0 -- bit
                              , @DeleteCleanUpList = @DeleteCleanUpList               -- bit
                              , @DeleteRecord = @DeleteRecord                    -- bit
                              , @TableName =@TableName                       -- nvarchar(128)
                              , @TenantID = @TenantID                           -- int
                              , @RecordOID = @RecordOID                       -- uniqueidentifier
                              , @RecordID =@RecordID                         -- nvarchar(8)
                              , @BatchNo = @BatchNo               -- int
                              , @orgAniceUser = @orgAniceUser                     -- nvarchar(32)
                              , @AddressID = NULL                        -- nvarchar(8)
                              , @CleanUpAllAddressContacts = 0       -- bit
                              , @ActionType = N''Clean''                       -- nvarchar(16)




	SET @BatchNo = 0;
RETURN @BatchNo;




END;



'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [privacy].[ufnEmployeeListToCleanUp]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[privacy].[ufnEmployeeListToCleanUp]') AND (type = 'IF' OR type = 'FN' OR type = 'TF'))
EXEC sp_executesql N'

-- =============================================
-- Author:		Falk Wegener
-- Create date: 2018-03-14
-- Description:	Returning a table with all Employee records wich match the criteria
--				PDP is the abrevation used for PrivateDataProtection
--				If no permission is set (Feature not active), then You can just use the default value for the related parameter

-- Example: SELECT  COUNT(ucltcu.__Org_RecordNo) from privacy.ufnEmployeeListToCleanUp() AS ucltcu WHERE ucltcu.__Org_RecordNo = 66
--			''Default parameter value
--			SELECT * from privacy.ufnEmployeeListToCleanUp()
-- =============================================
CREATE FUNCTION [privacy].[ufnEmployeeListToCleanUp]
()
RETURNS TABLE
AS
RETURN
(
    SELECT  emp.__Org_RecordOID
          , emp.ID
          , emp.[Name]
          , emp.Vorname
          , ''PGM22000_PzeMitarbeiter'' AS TableName
    FROM    dbo.PGM22000_PzeMitarbeiter AS emp
    WHERE
            (
                (
                    (
                        emp.BereinigtAm IS NULL --column never used
                        OR  YEAR(emp.BereinigtAm) = 1899 --column used and date removed again by orgAnice application
                    )
                    AND emp.Austrittsdatum        <= GETDATE()
                    AND YEAR(emp.Austrittsdatum)  > 1899
                )
                AND
                (
                    emp.PrivateDatenBehalten      = 0
                    OR  emp.PrivateDatenBehalten IS NULL
                )
            )
            OR
            (
                (
                    (
                        emp.BereinigtAm IS NULL --column never used
                        OR  YEAR(emp.BereinigtAm) = 1899 --column used and date removed again by orgAnice application
                    )
                    AND emp.Austrittsdatum        <= GETDATE()
                    AND YEAR(emp.Austrittsdatum)  > 1899
                )
                AND
                (
                    emp.PrivateDatenBehalten      = 0
                    OR  emp.PrivateDatenBehalten IS NULL
                )
            )
);




'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [privacy].[ufnAddressListToDelete]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[privacy].[ufnAddressListToDelete]') AND (type = 'IF' OR type = 'FN' OR type = 'TF'))
EXEC sp_executesql N'
-- =============================================
-- Author:		Falk Wegener
-- Create date: 2018-08-21
-- Description:	Returning a table with all Address records wich match the criteria
-- Example: SELECT * FROM privacy.ufnAddressListToDelete(2,2)
--			
--			
-- =============================================
CREATE FUNCTION [privacy].[ufnAddressListToDelete]
(
    @RententionTimeInMonths4Activities AS INT = 132 --11 years
  , @RententionTimeInMonths4Invoices AS   INT = 132 --11 years
  , @RententionTimeInMonths4Projects AS   INT = 132 --11 years
  , @RententionTimeInMonths4Addresses AS   INT = 132 --11 years
)
RETURNS TABLE
AS
RETURN
(
    SELECT  DISTINCT
            a.__Org_RecordOID
          , a.Name
          , a.Zusatz
          , a.ID
            -- ,ap.ID AS apid
          , (
                SELECT  MAX(r.Datum)
                FROM    dbo.Rechnungen    AS r
                    INNER JOIN
                        dbo.Kundennummern AS k
                            ON k.ID            = r.KundennummerID
                               AND  k.AdressID = a.ID
                WHERE   r.Datum <= DATEADD(MONTH, @RententionTimeInMonths4Invoices * (-1), GETDATE())
            ) AS MaxRechnungsdatum
          , (
                SELECT  MIN(r.Datum)
                FROM    dbo.Rechnungen    AS r
                    INNER JOIN
                        dbo.Kundennummern AS k
                            ON k.ID            = r.KundennummerID
                               AND  k.AdressID = a.ID
                WHERE   r.Datum <= DATEADD(MONTH, @RententionTimeInMonths4Invoices * (-1), GETDATE())
            ) AS MinRechnungsdatum
          , (
                SELECT  COUNT(r.Datum)
                FROM    dbo.Rechnungen    AS r
                    INNER JOIN
                        dbo.Kundennummern AS k
                            ON k.ID            = r.KundennummerID
                               AND  k.AdressID = a.ID
                WHERE   r.Datum <= DATEADD(MONTH, @RententionTimeInMonths4Invoices * (-1), GETDATE())
            ) AS AnzahlRechnungen
          , (
                SELECT  MAX(akt.Datum)
                FROM    dbo.Aktivitaeten AS akt
                    INNER JOIN
                        dbo.Vorgaenge    AS v
                            ON v.ID            = akt.VorgangID
                               AND  v.AdressID = a.ID
                WHERE   akt.Datum <= DATEADD(MONTH, @RententionTimeInMonths4Activities * -1, GETDATE())
            ) AS MaxAktivitaetsdatum
          , (
                SELECT  MIN(akt.Datum)
                FROM    dbo.Aktivitaeten AS akt
                    INNER JOIN
                        dbo.Vorgaenge    AS v
                            ON v.ID            = akt.VorgangID
                               AND  v.AdressID = a.ID
                WHERE   akt.Datum <= DATEADD(MONTH, @RententionTimeInMonths4Activities * -1, GETDATE())
            ) AS MinAktivitaetsdatum
          , (
                SELECT  COUNT(akt.Datum)
                FROM    dbo.Aktivitaeten AS akt
                    INNER JOIN
                        dbo.Vorgaenge    AS v
                            ON v.ID            = akt.VorgangID
                               AND  v.AdressID = a.ID
                WHERE   akt.Datum <= DATEADD(MONTH, @RententionTimeInMonths4Activities * -1, GETDATE())
            ) AS AnzahlAktivitaeten
    FROM    dbo.Adressen        AS a
        LEFT JOIN
            dbo.Ansprechpartner AS ap
                ON ap.AdressID = a.ID
    WHERE

        --Filter rules: no invoice younger than given parameter
            NOT EXISTS
    (
        SELECT  r.*
        FROM    dbo.Rechnungen    AS r
            INNER JOIN
                dbo.Kundennummern AS k
                    ON k.ID            = r.KundennummerID
                       AND  k.AdressID = a.ID
        WHERE   r.Datum >= DATEADD(MONTH, @RententionTimeInMonths4Invoices * (-1), GETDATE())
    )
            --Filter rules: no activity record younger than given parameter
            AND NOT EXISTS
    (
        SELECT  akt.*
        FROM    dbo.Aktivitaeten AS akt
            INNER JOIN
                dbo.Vorgaenge    AS v
                    ON v.ID            = akt.VorgangID
                       AND  v.AdressID = a.ID
        WHERE   akt.Datum >= DATEADD(MONTH, @RententionTimeInMonths4Activities * -1, GETDATE())
    )
            --Filter rules: no open project than given parameter for Adress or ContactID
            AND NOT EXISTS
    (
        SELECT --pa.AdressID, pa.AnsprechpartnerID,
                prj.*
        FROM    dbo.PRJ_Projekte        AS prj
            INNER JOIN
                dbo.PRJ_ProjektAdressen AS pa
                    ON pa.ProjektID = prj.ID
            INNER JOIN
                dbo.Adressen            AS a1
                    ON a1.ID        = pa.AdressID
                       AND  a1.ID   = a.ID
            INNER JOIN
                dbo.Ansprechpartner     AS ap1
                    ON ap1.ID       = pa.AnsprechpartnerID
                       AND  ap1.ID  = ap.ID
        WHERE   prj.ProjektEnde >= DATEADD(MONTH, @RententionTimeInMonths4Projects * -1, GETDATE())
    )
            --Filter rules: only addresses with no records in tables Aktivitaeten, Kundennummern, Vorgaengen and Ccreation date older than given parameter
            or
            (
                NOT EXISTS
    (
        SELECT  akt.*
        FROM    dbo.Aktivitaeten AS akt
            INNER JOIN
                dbo.Vorgaenge    AS v
                    ON v.ID = akt.VorgangID
        WHERE   v.AdressID = a.ID
    )
                AND NOT EXISTS
    (
        SELECT  k.* FROM dbo.Kundennummern AS k WHERE   k.AdressID = a.ID
    )
                AND NOT EXISTS
    (
        SELECT  ap1.* FROM  dbo.Ansprechpartner AS ap1 WHERE ap1.AdressID = a.ID
    )
                AND NOT EXISTS
    (
        SELECT  pa.* FROM   dbo.PRJ_ProjektAdressen AS pa WHERE pa.ProjektID = a.ID
    )
                AND a.AngelegtAm <= DATEADD(MONTH, @RententionTimeInMonths4Addresses * -1, GETDATE())
            )
);






'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [privacy].[GenerateDeleteTriggerForOrgDocumentsTable]'
GO
IF OBJECT_ID(N'[privacy].[GenerateDeleteTriggerForOrgDocumentsTable]', 'P') IS NULL
EXEC sp_executesql N'CREATE PROCEDURE [privacy].[GenerateDeleteTriggerForOrgDocumentsTable] AS
BEGIN
BEGIN TRY
    BEGIN TRANSACTION;



    DECLARE @JobList AS TABLE
    (
        ID                     INT            IDENTITY
      , DeleteTriggerStatement NVARCHAR(4000) NOT NULL
      , CreateTriggerStatement NVARCHAR(MAX)  NOT NULL
    );

    INSERT  @JobList
    (
        DeleteTriggerStatement
      , CreateTriggerStatement
    )
    SELECT  N''

	IF (OBJECT_ID(N''''[dbo].[TRG_'' + t.name + ''_DeleteOrgDocumentRecords]'''') IS NOT NULL)
BEGIN
      DROP TRIGGER [dbo].[TRG_'' + t.name + ''_DeleteOrgDocumentRecords];
END; ''              AS DeleteTriggerStatement
, N''CREATE TRIGGER TRG_'' + t.name + ''_DeleteOrgDocumentRecords ON '' + ObjectName + ''
            AFTER DELETE 
            AS 
            BEGIN
    SET NOCOUNT ON;
	DECLARE @parent_id INT;
    DELETE  FROM dbo.__Org_Documents
    WHERE
        EXISTS
    (
'' +

--SELECT
--SELECT  * FROM  Deleted AS d WHERE tt.dokument = DocOid OR tt.test2 = DocOid
--t.name AS tablename, 
''SELECT '' + STUFF((
                      SELECT    '', '' + QUOTENAME(c.name)
                      FROM      sys.columns AS c
                      WHERE
                                c.object_id          = t.object_id
                                AND c.system_type_id = 36
                                AND c.name           <> ''__Org_RecordOID''
                                AND LEFT(t.name, 6)  <> ''__Org_''
                      FOR XML PATH(''''), TYPE
                  ).value(''.'', ''NVARCHAR(MAX)'')
                , 1
                , 2
                , ''''
                 ) + '' FROM  Deleted AS d WHERE '' + STUFF((
                                                              SELECT    '' OR d.'' + QUOTENAME(c.name) + '' = [DOCOID]''
                                                              FROM      sys.columns AS c
                                                              WHERE
                                                                        c.object_id          = t.object_id
                                                                        AND c.system_type_id = 36
                                                                        AND c.name           <> ''__Org_RecordOID''
                                                                        AND LEFT(t.name, 6)  <> ''__Org_''
                                                              FOR XML PATH(''''), TYPE
                                                          ).value(''.'', ''NVARCHAR(MAX)'')
                                                        , 1
                                                        , 3
                                                        , ''''
                                                         ) + ''''
  +       '' );  END;  '' AS CreateTriggerStatement
    FROM    sys.tables                                                                    AS t
        CROSS APPLY
            (SELECT QUOTENAME(OBJECT_SCHEMA_NAME(t.object_id)) + ''.'' + QUOTENAME(t.name)) o(ObjectName)
    WHERE
            EXISTS
    (
        SELECT  1
        FROM    sys.columns AS c
        WHERE
            --c.Name = ''LastUpdate'' 
                c.system_type_id        = 36
                AND c.name              <> ''__Org_RecordOID''
                AND LEFT(t.name, 6)     <> ''__Org_''
                AND LEFT(ObjectName, 5) = ''[dbo]''
                AND c.object_id         = t.object_id
    );


    --SELECT  * FROM  @JobList AS jl;


    DECLARE @StatementMax  INT
          , @statementMin  INT
          , @isTest        TINYINT      = 0 
          , @SQLStatement1 NVARCHAR(MAX)
          , @SQLStatement2 NVARCHAR(MAX);


    SELECT  @StatementMax = MAX(ID), @statementMin = MIN(ID)FROM @JobList;
    IF @isTest = 1
    BEGIN
        SELECT  *, @StatementMax AS MaxVal, @statementMin AS MinVal FROM @JobList;
    END;
    -- Start the Loop
    WHILE @StatementMax >= @statementMin
    BEGIN
        SELECT  @SQLStatement1 = DeleteTriggerStatement
              , @SQLStatement2 = CreateTriggerStatement
        FROM    @JobList
        WHERE   ID = @statementMin; -- Get the SQL from the table 

        IF @isTest = 1
        BEGIN
            SELECT  ''I am executing: '' + @SQLStatement1 AS theSqlBeingRun
                  , GETDATE()
                  , @statementMin
                  , @StatementMax;
        END;
        IF @isTest = 1
        BEGIN
            SELECT  ''I am executing: '' + @SQLStatement2 AS theSqlBeingRun
                  , GETDATE()
                  , @statementMin
                  , @StatementMax;
        END;
        ELSE
        BEGIN
			--PRINT ''Executing statement1''
			--PRINT @SQLStatement1
            EXECUTE sp_executesql @SQLStatement1;   -- Execute the SQL 
			--PRINT ''Executing statement2''
			--PRINT @SQLStatement2
          --2019-01-24 FW disabled trigger creating, we dont need it anymore. We just keep the remove part for older databases
		  -- EXECUTE sp_executesql @SQLStatement2;   -- Execute the SQL 
        END;
        DELETE  FROM @JobList WHERE ID = @statementMin; -- Delete the statement just run from the table
        SELECT  @statementMin = MIN(ID)FROM @JobList;   -- Update to the next RID
        IF @isTest = 1 BEGIN
                           SELECT   * FROM  @JobList;
        END; -- List the Jobs which are still in the table

    END;


    COMMIT;
END TRY
BEGIN CATCH

    IF @@TRANCOUNT > 0 
	BEGIN
	PRINT ''failed''	
	ROLLBACK
	end
	;
END CATCH;
END

'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating [privacy].[spDeleteRecordsInDetailTable]'
GO
IF OBJECT_ID(N'[privacy].[spDeleteRecordsInDetailTable]', 'P') IS NULL
EXEC sp_executesql N'


-- =============================================
-- Author:		Falk Wegener
-- Create date: 2018-03-26
-- Description:	Deleting records from a detail table based on mastertable FK key. 
-- =============================================
CREATE PROCEDURE [privacy].[spDeleteRecordsInDetailTable]
	@MasterTableName AS NVARCHAR(128),
    @MasterTableFKName AS NVARCHAR(128),
	@MasterTableFKValue AS NVARCHAR(10),
	@MasterTableIDColumnName AS NVARCHAR(128),
    @DetailTableName AS NVARCHAR(128),
    @DetailTableFKCoumnNameForMasterTable AS NVARCHAR(128),
	@RecordsDeleted INT OUTPUT
AS
BEGIN
    -- SET NOCOUNT ON added to prevent extra result sets from
    -- interfering with SELECT statements.
    --SET NOCOUNT OFF;
	DECLARE @sqlCmd AS NVARCHAR(4000) = ''''

	--parse delete statement like
	--DELETE t1 FROM dbo.Aktivitaeten t1 
	--	INNER JOIN  dbo.Vorgaenge AS t2 ON t2.ID = t1.VorgangID
	--	WHERE t2.AdressID =''0000000
	SET @sqlCmd = ''DELETE t1 FROM '' + @DetailTableName + '' t1 '' 
	--''+='' add String to string
	SET @sqlCmd += '' INNER JOIN  '' + @MasterTableName + '' AS t2 ON t2.'' + @MasterTableIDColumnName  +'' = t1.'' +@DetailTableFKCoumnNameForMasterTable + '' ''
	SET @sqlCmd +=  ''WHERE t2.'' + @MasterTableFKName + '' = '''''' + @MasterTableFKValue + '''''' ''
	
	PRINT @sqlCmd

	EXEC sys.sp_executesql @sqlCmd

	SET @RecordsDeleted= @@ROWCOUNT
	RETURN @RecordsDeleted
	END;



'
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Adding constraints to [dbo].[Blacklist]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.check_constraints WHERE object_id = OBJECT_ID(N'[dbo].[__OrgC_Blacklist___Org_Tenants]', 'C') AND parent_object_id = OBJECT_ID(N'[dbo].[Blacklist]', 'U'))
ALTER TABLE [dbo].[Blacklist] ADD CONSTRAINT [__OrgC_Blacklist___Org_Tenants] CHECK NOT FOR REPLICATION ((coalesce([Blacklist].[__Org_Tenants],(0)&((-2)))<>(0)))
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Adding constraints to [dbo].[DSGVOFeldKonfigurationen]'
GO
IF NOT EXISTS (SELECT 1 FROM sys.check_constraints WHERE object_id = OBJECT_ID(N'[dbo].[__OrgC_DSGVOFeldKonfigurationen___Org_Tenants]', 'C') AND parent_object_id = OBJECT_ID(N'[dbo].[DSGVOFeldKonfigurationen]', 'U'))
ALTER TABLE [dbo].[DSGVOFeldKonfigurationen] ADD CONSTRAINT [__OrgC_DSGVOFeldKonfigurationen___Org_Tenants] CHECK NOT FOR REPLICATION ((coalesce([DSGVOFeldKonfigurationen].[__Org_Tenants],(0)&((-2)))<>(0)))
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
IF NOT EXISTS (SELECT 1 FROM sys.check_constraints WHERE object_id = OBJECT_ID(N'[dbo].[__OrgC_DSGVOFeldKonfigurationen_TabellenName]', 'C') AND parent_object_id = OBJECT_ID(N'[dbo].[DSGVOFeldKonfigurationen]', 'U'))
ALTER TABLE [dbo].[DSGVOFeldKonfigurationen] ADD CONSTRAINT [__OrgC_DSGVOFeldKonfigurationen_TabellenName] CHECK NOT FOR REPLICATION ((len(coalesce([DSGVOFeldKonfigurationen].[TabellenName],'') COLLATE Latin1_General_CS_AS+'-')-(1)>(0)))
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
IF NOT EXISTS (SELECT 1 FROM sys.check_constraints WHERE object_id = OBJECT_ID(N'[dbo].[__OrgC_DSGVOFeldKonfigurationen_EigenschaftenName]', 'C') AND parent_object_id = OBJECT_ID(N'[dbo].[DSGVOFeldKonfigurationen]', 'U'))
ALTER TABLE [dbo].[DSGVOFeldKonfigurationen] ADD CONSTRAINT [__OrgC_DSGVOFeldKonfigurationen_EigenschaftenName] CHECK NOT FOR REPLICATION ((len(coalesce([DSGVOFeldKonfigurationen].[EigenschaftenName],'') COLLATE Latin1_General_CS_AS+'-')-(1)>(0)))
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'Creating extended properties'
GO
BEGIN TRY
	IF NOT EXISTS (SELECT 1 FROM fn_listextendedproperty(N'MS_Description', 'SCHEMA', N'privacy', 'TABLE', N'CleanUpJobList_aid', NULL, NULL))
EXEC sp_addextendedproperty N'MS_Description', N'Aid table for storing the jobs temporarily. If everything is working properly, then this table is empty after every job has been processed. Records contain SPID to distinguish the records from each orgAnice user.', 'SCHEMA', N'privacy', 'TABLE', N'CleanUpJobList_aid', NULL, NULL
END TRY
BEGIN CATCH
	DECLARE @msg nvarchar(max);
	DECLARE @severity int;
	DECLARE @state int;
	SELECT @msg = ERROR_MESSAGE(), @severity = ERROR_SEVERITY(), @state = ERROR_STATE();
	RAISERROR(@msg, @severity, @state);

	SET NOEXEC ON
END CATCH
GO
BEGIN TRY
	IF NOT EXISTS (SELECT 1 FROM fn_listextendedproperty(N'MS_Description', 'SCHEMA', N'org', 'PROCEDURE', N'spWriteToLogTable', NULL, NULL))
EXEC sp_addextendedproperty N'MS_Description', N'Proc to ease and abstract the coding for adding a new record into the Logging table and removing old records too.', 'SCHEMA', N'org', 'PROCEDURE', N'spWriteToLogTable', NULL, NULL
END TRY
BEGIN CATCH
	DECLARE @msg nvarchar(max);
	DECLARE @severity int;
	DECLARE @state int;
	SELECT @msg = ERROR_MESSAGE(), @severity = ERROR_SEVERITY(), @state = ERROR_STATE();
	RAISERROR(@msg, @severity, @state);

	SET NOEXEC ON
END CATCH
GO

COMMIT TRANSACTION
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
DECLARE @Success AS BIT
SET @Success = 1
SET NOEXEC OFF
IF (@Success = 1) PRINT 'The database update for part 03 succeeded'
ELSE BEGIN
	IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION
	PRINT 'The database update for part 03 failed'
END
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO

DECLARE @Success AS BIT
SET @Success = 1
SET NOEXEC OFF
IF (@Success = 1) PRINT 'The database update  for part 03 succeeded'
ELSE BEGIN
	IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION
	PRINT 'The database update for part 03 failed'
END
GO

-- Part 04 - execute sp in order to remove old trigger from user tables
DECLARE	@return_value int

EXEC	@return_value = [privacy].[GenerateDeleteTriggerForOrgDocumentsTable]

go

--SELECT	'Return Value' = @return_value

/* **
*:*:*:MbfHyGTLmPnWBxvCzMWy7Sz4W0bNRIyE:mHZXe3lpIdMYq1JeOoCtIZGY9XLOXQJIFjqq8zHfaBE=
*/
